@@ -2,6 +2,7 @@
     Utilities for parsing Character Separated Value files (CSV)
 
     Copyright (C) 2002 Alex Mottram (geo_alexm at cox-internet.com)
+    Copyright (C) 2002-2005 Robert Lipe
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 #include "defs.h"
 #include "csv_util.h"
 #include "grtcirc.h"
+#include "strptime.h"
 
 #define MYNAME "CSV_UTIL"
 
@@ -39,6 +41,10 @@
 extern char *xcsv_urlbase;
 extern char *prefer_shortnames;
 
+extern const char *gs_get_container(geocache_container t);
+extern geocache_type gs_mktype(const char *t);
+extern geocache_container gs_mkcont(const char *t);
+
 static double pathdist = 0;
 static double oldlon = 999;
 static double oldlat = 999;
@@ -77,14 +83,13 @@ csv_stringclean(const char *string, const char *chararray)
         while (*p1) {
             if (*cp == *p1) {
                 /* we don't want this character! */
-                strncpy(p1, p1 + 1, (p2 - p1));
+		memmove(p1, p1 + 1, (p2 - p1));
                 p1[p2 - p1] = '\0';
             }
             p1++;
         }
         cp++;
     }
-    
     return (tmp);
 }
 
@@ -146,7 +151,8 @@ csv_stringtrim(const char *string, const char *enclosure, int strip_max)
     }
 
     /* copy what's left over back into tmp. */
-    strncpy(tmp, p1, (p2 - p1) + 1);
+    memmove(tmp, p1, (p2 - p1) + 1);
+
     tmp[(p2 - p1) + 1] = '\0';
 
     return (tmp);
@@ -180,6 +186,17 @@ csv_lineparse(const char *stringstart, const char *delimited_by,
     if (strcmp(delimited_by, "\\w") == 0)
         hyper_whitespace_delimiter = 1;
 
+    /*
+     * This is tacky.  Our "csv" format is actually "commaspace" format.
+     * Changing that causes unwanted churn, but it also makes "real" 
+     * comma separated data (such as likely to be produced by Excel, etc.) 
+     * unreadable.   So we silently change it here on a read and let the
+     * whitespace eater consume the space.
+     */
+    if (strcmp(delimited_by, ", ") == 0) {
+	delimited_by = ",";
+    }
+
     if (!p) {
 	/* first pass thru */
 	p =  stringstart;
@@ -322,16 +339,19 @@ decdir_to_dec(const char * decdir)
  * human_to_dec() - convert a "human-readable" lat and/or lon to decimal
  * usage: human_to_dec( "N 41 09.12' W 085 09.36'", &lat, &lon );
  *        human_to_dec( "41 9 5.652 N", &lat, &lon );
+ *        
+ *        which: 0-no preference    1-prefer lat    2-prefer lon
  *****************************************************************************/
 
 static void
-human_to_dec( const char *instr, double *outlat, double *outlon )
+human_to_dec( const char *instr, double *outlat, double *outlon, int which )
 {
     double unk[3] = {999,999,999};
     double lat[3] = {999,999,999};
     double lon[3] = {999,999,999};
     int    latsign = 0;
     int    lonsign = 0;
+    int    unksign = 1;
    
     const char *cur;
     double *numres = unk;
@@ -388,6 +408,10 @@ human_to_dec( const char *instr, double *outlat, double *outlon )
 		numres[numind] = atof(cur);
 		while (cur && *cur && strchr("1234567890.",*cur)) cur++;
 		break;
+	    case '-':
+		unksign = -1;
+		cur++;
+		break;
 	    default:
 		if (numres[numind] != 999) {
     		    numind++;
@@ -401,6 +425,17 @@ human_to_dec( const char *instr, double *outlat, double *outlon )
 	}
     }
     
+    if ( lat[0] == 999 && lon[0] == 999 ) {
+	if ( which == 1 ) {
+	    lat[0] = unk[0]; lat[1] = unk[1]; lat[2] = unk[2];
+	    latsign = unksign;
+	}
+	else if ( which == 2 ) {
+	    lon[0] = unk[0]; lon[1] = unk[1]; lon[2] = unk[2];
+	    lonsign = unksign;
+	}
+    }
+    
     if ( outlat ) {
 	if ( lat[0] != 999 ) *outlat = lat[0];
 	if ( lat[1] != 999 ) *outlat += lat[1]/60.0;
@@ -580,6 +615,123 @@ xcsv_epilogue_add(char *epilogue)
     xcsv_file.epilogue_lines++;
 }
 
+static
+time_t
+yyyymmdd_to_time(const char *s)
+{
+	int t = atol(s);
+	struct tm tm;
+
+	memset(&tm, 0, sizeof(tm));
+
+	tm.tm_mday = t % 100;
+	t = t / 100;
+	tm.tm_mon = t % 100 - 1;
+	t = t / 100;
+	tm.tm_year = t - 1900;
+	return mktime(&tm);
+}
+
+
+
+/*
+ * sscanftime - Parse a date buffer using strftime format
+ */
+static
+time_t
+sscanftime( const char *s, const char *format, const int gmt )
+{
+	struct tm stm = {0,0,0,0,0,0,0,0,0};
+	if ( strptime( s, format, &stm ) )
+	{
+		stm.tm_isdst = -1;
+		if (gmt)
+			return mkgmtime(&stm);
+		else
+			return mktime(&stm);		
+	}
+	
+	return -1;
+}
+static
+time_t
+addhms( const char *s, const char *format )
+{
+	time_t tt =0;
+	int  hour =0;
+	int  min  =0;
+	int  sec  =0;
+	char * ampm = NULL;
+	
+	ampm = xmalloc( strlen(s) );
+	if (sscanf(s, format, &hour, &min, &sec, ampm))
+	   tt = ((tolower(ampm[0])=='P')?43200:0)+3600*hour+60*min+sec;
+    	xfree(ampm);
+    	
+	return tt;
+}
+
+static 
+int 
+writetime(char * buff, size_t bufsize, const char * format, time_t t, int gmt )
+{
+	static struct tm * stmp;
+
+	if (gmt)
+		stmp = gmtime(&t);
+	else
+		stmp = localtime(&t);
+
+	return strftime(buff, bufsize, format, stmp );
+}
+
+static 
+int 
+writeisotime(char * buff, size_t bufsize, const char * format, time_t t)
+{
+	static struct tm * stmp;
+	char * ibuff = NULL;
+	int i;
+	
+	ibuff = xmalloc(bufsize);
+	stmp = gmtime(&t);
+	strftime(ibuff, bufsize, format, stmp );
+	i = snprintf(buff, bufsize, format, ibuff );
+	xfree(ibuff);
+	return i;
+}
+
+
+static 
+int 
+writehms(char * buff, size_t bufsize, const char * format, time_t t, int gmt )
+{
+	static struct tm * stmp;
+
+	if (gmt)
+		stmp = gmtime(&t);
+	else
+		stmp = localtime(&t);
+
+	return snprintf(buff, bufsize, format, 
+		stmp->tm_hour, stmp->tm_min, stmp->tm_sec, 
+		(stmp->tm_hour>=12?"PM":"AM") );
+}
+
+static 
+long 
+time_to_yyyymmdd(time_t t)
+{
+	long b;
+	struct tm *tm = gmtime(&t);
+
+	b = (1900 + tm->tm_year) * 10000 + 
+		(1 + tm->tm_mon) * 100 + 
+		tm->tm_mday;
+
+	return b;
+}
+
 /*****************************************************************************/
 /* xcsv_parse_val() - parse incoming data into the waypt structure.          */
 /* usage: xcsv_parse_val("-123.34", *waypt, *field_map)                      */
@@ -616,7 +768,7 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     } else
     if (strcmp(fmp->key, "ICON_DESCR") == 0) {
        wpt->icon_descr = csv_stringtrim(s, "", 0);
-       wpt->icon_descr_is_dynamic = 1;
+       wpt->wpt_flags.icon_descr_is_dynamic = 1;
     } else
 
     /* LATITUDE CONVERSIONS**************************************************/
@@ -631,10 +783,10 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     } else
     if (strcmp(fmp->key, "LAT_INT32DEG") == 0) {
        /* latitude as a 32 bit integer offset */
-       wpt->latitude = intdeg_to_dec(atof(s), 1);
+       wpt->latitude = intdeg_to_dec((int) atof(s), 1);
     } else
     if ( strcmp(fmp->key, "LAT_HUMAN_READABLE") == 0) {
-       human_to_dec( s, &wpt->latitude, &wpt->longitude );
+       human_to_dec( s, &wpt->latitude, &wpt->longitude, 1 );
     } else
     if ( strcmp(fmp->key, "LAT_NMEA") == 0) {
 	wpt->latitude = ddmm2degrees(wpt->latitude);
@@ -651,17 +803,17 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     } else
     if (strcmp(fmp->key, "LON_INT32DEG") == 0) {
        /* longitude as a 32 bit integer offset  */
-       wpt->longitude = intdeg_to_dec(atof(s), 0);
+       wpt->longitude = intdeg_to_dec((int) atof(s), 0);
     } else
     if ( strcmp(fmp->key, "LON_HUMAN_READABLE") == 0) {
-       human_to_dec( s, &wpt->latitude, &wpt->longitude );
+       human_to_dec( s, &wpt->latitude, &wpt->longitude, 2 );
     } else
     if ( strcmp(fmp->key, "LON_NMEA") == 0) {
 	wpt->latitude = ddmm2degrees(wpt->longitude);
     } else
     /* LAT AND LON CONVERSIONS ********************************************/
     if ( strcmp(fmp->key, "LATLON_HUMAN_READABLE") == 0) {
-       human_to_dec( s, &wpt->latitude, &wpt->longitude );
+       human_to_dec( s, &wpt->latitude, &wpt->longitude, 0 );
     } else
     /* DIRECTIONS **********************************************************/
     if (strcmp(fmp->key, "LAT_DIR") == 0) {
@@ -680,6 +832,14 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
        /* altitude in meters as a decimal value */
        wpt->altitude = atof(s);
     } else
+    
+    /* PATH CONVERSIONS ************************************************/
+    if (strcmp(fmp->key, "PATH_SPEED") == 0) {
+	wpt->speed = atof(s);
+    } else
+    if (strcmp(fmp->key, "PATH_COURSE") == 0) {
+	wpt->course = atof(s);
+    } else
 
     /* TIME CONVERSIONS ***************************************************/
     if (strcmp(fmp->key, "EXCEL_TIME") == 0) {
@@ -690,6 +850,27 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
        /* Time as time_t */
        wpt->creation_time = atol(s);
      } else
+    if (strcmp(fmp->key, "YYYYMMDD_TIME") == 0) {
+	wpt->creation_time = yyyymmdd_to_time(s);
+    } else
+    if (strcmp(fmp->key, "GMT_TIME") == 0) {
+	wpt->creation_time = sscanftime(s, fmp->printfc, 1);
+    } else
+    if (strcmp(fmp->key, "LOCAL_TIME") == 0) {
+	wpt->creation_time = sscanftime(s, fmp->printfc, 0);
+    } else
+    /* Useful when time and date are in separate fields 
+    	GMT / Local offset is handled by the two cases above */
+    if ((strcmp(fmp->key, "HMSG_TIME") == 0)||
+    	(strcmp(fmp->key, "HMSL_TIME") == 0) ) {
+	wpt->creation_time += addhms(s, fmp->printfc);
+    } else
+    if (strcmp(fmp->key, "ISO_TIME") == 0) {
+	wpt->creation_time = xml_parse_time(s);
+    } else
+	if (strcmp(fmp->key, "GEOCACHE_LAST_FOUND") == 0) {
+	wpt->gc_data.last_found = yyyymmdd_to_time(s);
+    } else
 
     /* GEOCACHING STUFF ***************************************************/
     if (strcmp(fmp->key, "GEOCACHE_DIFF") == 0) {
@@ -707,6 +888,41 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     if (strcmp(fmp->key, "GEOCACHE_CONTAINER") == 0) {
        wpt->gc_data.container = gs_mkcont(s);
     } else
+    if (strcmp(fmp->key, "GEOCACHE_HINT") == 0) {
+       wpt->gc_data.hint = csv_stringtrim(s, "", 0);
+    } else
+    if (strcmp(fmp->key, "GEOCACHE_PLACER") == 0) {
+       wpt->gc_data.placer = csv_stringtrim(s, "", 0);
+    } else
+	
+    /* GPS STUFF *******************************************************/
+    if (strcmp(fmp->key, "GPS_HDOP") == 0) {
+	 wpt->hdop = atof(s);
+    } else
+    if (strcmp(fmp->key, "GPS_VDOP") == 0) {
+	wpt->vdop = atof(s);
+    } else
+    if (strcmp(fmp->key, "GPS_PDOP") == 0) {
+        wpt->pdop = atof(s);
+    } else
+    if (strcmp(fmp->key, "GPS_SAT") == 0) {
+	wpt->sat = atoi(s);
+    } else
+    if (strcmp(fmp->key, "GPS_FIX") == 0) {
+	wpt->fix = atoi(s)-1;
+	if ( wpt->fix < fix_2d) {
+	if (!case_ignore_strcmp(s, "none"))
+		wpt->fix = fix_none;
+	else if (!case_ignore_strcmp(s, "dgps"))
+		wpt->fix = fix_dgps;
+	else if (!case_ignore_strcmp(s, "pps"))
+		wpt->fix = fix_pps;
+	else
+		wpt->fix = fix_unknown;
+	}
+    } else
+	
+    /* OTHER STUFF ***************************************************/
     if ( strcmp( fmp->key, "PATH_DISTANCE_MILES") == 0) {
        /* Ignored on input */
     } else
@@ -759,7 +975,7 @@ xcsv_data_read(void)
         }
 
         if (strlen(buff)) {
-            wpt_tmp = xcalloc(sizeof(*wpt_tmp), 1);
+            wpt_tmp = waypt_new();
 
             s = buff;
             s = csv_lineparse(s, xcsv_file.field_delimiter, "", linecount);
@@ -794,7 +1010,7 @@ xcsv_data_read(void)
 }
 
 static void
-xcsv_resetpathlen()
+xcsv_resetpathlen(const route_head *head)
 {
     pathdist = 0;
     oldlat = 999;
@@ -824,15 +1040,15 @@ xcsv_waypt_pr(const waypoint *wpt)
     oldlon = wpt->longitude;
     oldlat = wpt->latitude;
 
-    if (strcmp(xcsv_file.field_delimiter, "\\w") == 0)
+    if (xcsv_file.field_delimiter && strcmp(xcsv_file.field_delimiter, "\\w") == 0)
         write_delimiter = " ";
     else
         write_delimiter = xcsv_file.field_delimiter;
     
     if ((! wpt->shortname) || (global_opts.synthesize_shortnames)) {
         if (wpt->description) {
-            if (global_opts.synthesize_shortnames)
-                shortname = mkshort(xcsv_file.mkshort_handle, wpt->description);
+            if (global_opts.synthesize_shortnames) 
+		shortname = mkshort_from_wpt(xcsv_file.mkshort_handle, wpt);
             else
                 shortname = csv_stringclean(wpt->description, xcsv_file.badchars);
         } else {
@@ -856,10 +1072,8 @@ xcsv_waypt_pr(const waypoint *wpt)
 	    if (description) {
 		    xfree(description);
 	    }
-	    description = shortname;
-    }
-
-    if (description) {
+ 	    description = shortname;
+    } else if (description) {
 	    char *odesc = description;
 	    description = str_utf8_to_ascii(odesc);
 	    xfree(odesc);
@@ -867,25 +1081,31 @@ xcsv_waypt_pr(const waypoint *wpt)
 
     i = 0;
     QUEUE_FOR_EACH(xcsv_file.ofield, elem, tmp) {
+	char *obuff;
         fmp = (field_map_t *) elem;
 
         if (i != 0) 
             fprintf (xcsv_file.xcsvfp, write_delimiter);
 
         i++;
-
+#define writebuff(b, fmt, data) snprintf(b, sizeof(b), fmt, data)
         if (strcmp(fmp->key, "IGNORE") == 0) {
             /* IGNORE -- Write the char printf conversion */
-            sprintf(buff, fmp->printfc, "");
+            writebuff(buff, fmp->printfc, "");
         } else
         if (strcmp(fmp->key, "INDEX") == 0) {
-            sprintf(buff, fmp->printfc, waypt_out_count + atoi(fmp->val));
+            writebuff(buff, fmp->printfc, waypt_out_count + atoi(fmp->val));
         } else
         if (strcmp(fmp->key, "CONSTANT") == 0) {
-            sprintf(buff, fmp->printfc, fmp->val);
+	    const char *cp = xcsv_get_char_from_constant_table(fmp->val);
+	    if (cp) {
+                writebuff(buff, fmp->printfc, cp);
+	    } else {
+		writebuff(buff, fmp->printfc, fmp->val);
+	    }
         } else
         if (strcmp(fmp->key, "SHORTNAME") == 0) {
-            sprintf(buff, fmp->printfc, 
+            writebuff(buff, fmp->printfc, 
                 (shortname && *shortname) ? shortname : fmp->val);
         } else
         if (strcmp(fmp->key, "ANYNAME") == 0) {
@@ -904,16 +1124,16 @@ xcsv_waypt_pr(const waypoint *wpt)
                 anyname = xstrdup(shortname);
 	    }
 
-            sprintf(buff, fmp->printfc, anyname);
+            writebuff(buff, fmp->printfc, anyname);
             
             xfree(anyname);
         } else
         if (strcmp(fmp->key, "DESCRIPTION") == 0) {
-            sprintf(buff, fmp->printfc, 
+            writebuff(buff, fmp->printfc, 
                 (description && *description) ? description : fmp->val);
         } else
         if (strcmp(fmp->key, "NOTES") == 0) {
-	    sprintf(buff, fmp->printfc, 
+	    writebuff(buff, fmp->printfc, 
 	        (wpt->notes && *wpt->notes) ? wpt->notes : fmp->val);
         } else
         if (strcmp(fmp->key, "URL") == 0) {
@@ -923,16 +1143,16 @@ xcsv_waypt_pr(const waypoint *wpt)
 		off = strlen(xcsv_urlbase);
 	    }
 	    if (wpt->url)
-		sprintf(buff + off, fmp->printfc, wpt->url);
+		snprintf(buff + off, sizeof(buff) - off, fmp->printfc, wpt->url);
 	    else
 		strcpy(buff, (fmp->val && *fmp->val) ? fmp->val : "\"\"");
         } else
         if (strcmp(fmp->key, "URL_LINK_TEXT") == 0) {
-            sprintf(buff, fmp->printfc, 
+            snprintf(buff, sizeof(buff), fmp->printfc, 
                 (wpt->url_link_text && *wpt->url_link_text) ? wpt->url_link_text : fmp->val);
         } else
         if (strcmp(fmp->key, "ICON_DESCR") == 0) {
-            sprintf(buff, fmp->printfc, 
+            writebuff(buff, fmp->printfc, 
                 (wpt->icon_descr && *wpt->icon_descr) ? 
                 wpt->icon_descr : fmp->val);
         } else
@@ -940,51 +1160,51 @@ xcsv_waypt_pr(const waypoint *wpt)
         /* LATITUDE CONVERSION***********************************************/
         if (strcmp(fmp->key, "LAT_DECIMAL") == 0) {
             /* latitude as a pure decimal value */
-            sprintf(buff, fmp->printfc, wpt->latitude);
+            writebuff(buff, fmp->printfc, wpt->latitude);
         } else
         if (strcmp(fmp->key, "LAT_DECIMALDIR") == 0) {
             /* latitude as a decimal value with N/S after it */
-            sprintf(buff, fmp->printfc, fabs(wpt->latitude), 
+            snprintf(buff, sizeof(buff), fmp->printfc, fabs(wpt->latitude), 
               LAT_DIR(wpt->latitude));
         } else
         if (strcmp(fmp->key, "LAT_DIRDECIMAL") == 0) {
             /* latitude as a decimal value with N/S before it */
-            sprintf(buff, fmp->printfc, 
+            snprintf(buff, sizeof(buff), fmp->printfc, 
               LAT_DIR(wpt->latitude),
               fabs(wpt->latitude));
         } else
         if (strcmp(fmp->key, "LAT_INT32DEG") == 0) {
             /* latitude as an integer offset from 0 degrees */
-            sprintf(buff, fmp->printfc,
+            writebuff(buff, fmp->printfc,
               dec_to_intdeg(wpt->latitude, 1));
         } else
 	if (strcmp(fmp->key, "LAT_HUMAN_READABLE") == 0) {
 	    dec_to_human( buff, fmp->printfc, "SN", wpt->latitude );
 	} else
 	if (strcmp(fmp->key, "LAT_NMEA") == 0) {
-		sprintf(buff, fmp->printfc, degrees2ddmm(wpt->latitude));
+		writebuff(buff, fmp->printfc, degrees2ddmm(wpt->latitude));
 	} else
 
         /* LONGITUDE CONVERSIONS*********************************************/
         if (strcmp(fmp->key, "LON_DECIMAL") == 0) {
             /* longitude as a pure decimal value */
-            sprintf(buff, fmp->printfc, wpt->longitude);
+            writebuff(buff, fmp->printfc, wpt->longitude);
         } else
         if (strcmp(fmp->key, "LON_DECIMALDIR") == 0) {
             /* latitude as a decimal value with N/S after it */
-            sprintf(buff, fmp->printfc,
+            snprintf(buff, sizeof(buff),  fmp->printfc,
               fabs(wpt->longitude), 
               LON_DIR(wpt->longitude));
         } else
         if (strcmp(fmp->key, "LON_DIRDECIMAL") == 0) {
             /* latitude as a decimal value with N/S before it */
-            sprintf(buff, fmp->printfc,
+            snprintf(buff, sizeof(buff), fmp->printfc,
               LON_DIR(wpt->longitude),
               fabs(wpt->longitude));
         } else
         if (strcmp(fmp->key, "LON_INT32DEG") == 0) {
             /* longitudee as an integer offset from 0 degrees */
-            sprintf(buff, fmp->printfc,
+            writebuff(buff, fmp->printfc,
               dec_to_intdeg(wpt->longitude, 0));
         } else
 	if (strcmp(fmp->key, "LON_HUMAN_READABLE") == 0) {
@@ -997,85 +1217,160 @@ xcsv_waypt_pr(const waypoint *wpt)
 			    wpt->longitude );
 	} else
 	if (strcmp(fmp->key, "LON_NMEA") == 0) {
-		sprintf(buff, fmp->printfc, degrees2ddmm(wpt->longitude));
+		writebuff(buff, fmp->printfc, degrees2ddmm(wpt->longitude));
 	} else
 
         /* DIRECTIONS *******************************************************/
         if (strcmp(fmp->key, "LAT_DIR") == 0) {
             /* latitude N/S as a char */
-            sprintf(buff, fmp->printfc,
+            writebuff(buff, fmp->printfc,
             LAT_DIR(wpt->latitude));
         } else
         if (strcmp(fmp->key, "LON_DIR") == 0) {
             /* longitude E/W as a char */
-            sprintf(buff, fmp->printfc,
+            writebuff(buff, fmp->printfc,
               LON_DIR(wpt->longitude));
         } else
 
         /* ALTITUDE CONVERSIONS**********************************************/
         if (strcmp(fmp->key, "ALT_FEET") == 0) {
             /* altitude in feet as a decimal value */
-            sprintf(buff, fmp->printfc,
+            writebuff(buff, fmp->printfc,
               (wpt->altitude * 3.2808));
         } else
         if (strcmp(fmp->key, "ALT_METERS") == 0) {
             /* altitude in meters as a decimal value */
-            sprintf(buff, fmp->printfc,
+            writebuff(buff, fmp->printfc,
               wpt->altitude);
         } else
 		
         /* DISTANCE CONVERSIONS**********************************************/
 	if (strcmp(fmp->key, "PATH_DISTANCE_MILES") == 0) {
             /* path (route/track) distance in miles */
-            sprintf( buff, fmp->printfc, pathdist );
+            writebuff( buff, fmp->printfc, pathdist );
 	} else
 	if (strcmp(fmp->key, "PATH_DISTANCE_KM") == 0) {
             /* path (route/track) distance in  */
-            sprintf( buff, fmp->printfc, pathdist * 5280*12*2.54/100/1000 );
+            writebuff( buff, fmp->printfc, pathdist * 5280*12*2.54/100/1000 );
+	} else
+	if (strcmp(fmp->key, "PATH_SPEED") == 0) {
+            writebuff( buff, fmp->printfc, wpt->speed );
+	} else
+	if (strcmp(fmp->key, "PATH_COURSE") == 0) {
+            writebuff( buff, fmp->printfc, wpt->course );
 	} else
 
         /* TIME CONVERSIONS**************************************************/
         if (strcmp(fmp->key, "EXCEL_TIME") == 0) {
             /* creation time as an excel (double) time */
-            sprintf(buff, fmp->printfc, TIMET_TO_EXCEL(wpt->creation_time));
+            writebuff(buff, fmp->printfc, TIMET_TO_EXCEL(wpt->creation_time));
         } else
         if (strcmp(fmp->key, "TIMET_TIME") == 0) {
             /* time as a time_t variable */
-            sprintf(buff, fmp->printfc, wpt->creation_time);
+            writebuff(buff, fmp->printfc, wpt->creation_time);
         } else
+        if (strcmp(fmp->key, "YYYYMMDD_TIME") == 0) {
+	    writebuff(buff, fmp->printfc, time_to_yyyymmdd(wpt->creation_time));
+	} else
+	if (strcmp(fmp->key, "GMT_TIME") == 0) {
+	    writetime(buff, sizeof buff, fmp->printfc, wpt->creation_time, 1 );
+	} else
+        if (strcmp(fmp->key, "LOCAL_TIME") == 0) {
+            writetime(buff, sizeof buff, fmp->printfc, wpt->creation_time, 0 );
+	} else
+        if (strcmp(fmp->key, "HMSG_TIME") == 0) {
+            writehms(buff, sizeof buff, fmp->printfc, wpt->creation_time, 1 );
+	} else
+        if (strcmp(fmp->key, "HMSL_TIME") == 0) {
+            writehms(buff, sizeof buff, fmp->printfc, wpt->creation_time, 0 );
+	} else
+	if (strcmp(fmp->key, "ISO_TIME") == 0) {
+            writetime(buff, sizeof buff, "%Y-%m-%dT%H:%M:%SZ", wpt->creation_time, 1 );
+	} else
+        if (strcmp(fmp->key, "GEOCACHE_LAST_FOUND") == 0) {
+	    writebuff(buff, fmp->printfc, time_to_yyyymmdd(wpt->gc_data.last_found));
+	} else
 
         /* GEOCACHE STUFF **************************************************/
         if (strcmp(fmp->key, "GEOCACHE_DIFF") == 0) {
             /* Geocache Difficulty as a double */
-            sprintf(buff, fmp->printfc, wpt->gc_data.diff / 10.0);
+            writebuff(buff, fmp->printfc, wpt->gc_data.diff / 10.0);
         } else
         if (strcmp(fmp->key, "GEOCACHE_TERR") == 0) {
             /* Geocache Terrain as a double */
-            sprintf(buff, fmp->printfc, wpt->gc_data.terr / 10.0);
+            writebuff(buff, fmp->printfc, wpt->gc_data.terr / 10.0);
         } else
         if (strcmp(fmp->key, "GEOCACHE_CONTAINER") == 0) {
             /* Geocache Container */
-            sprintf(buff, fmp->printfc, gs_get_container(wpt->gc_data.container));
+            writebuff(buff, fmp->printfc, gs_get_container(wpt->gc_data.container));
 	} else
 	if (strcmp(fmp->key, "GEOCACHE_TYPE") == 0) {
             /* Geocache Type */
-            sprintf(buff, fmp->printfc, gs_get_cachetype(wpt->gc_data.type));
-        } else {
-           /* this should probably never happen */
+            writebuff(buff, fmp->printfc, gs_get_cachetype(wpt->gc_data.type));
+        } else 
+	if (strcmp(fmp->key, "GEOCACHE_HINT") == 0) {
+	    writebuff(buff, fmp->printfc, NONULL(wpt->gc_data.hint));
+        } else 
+	if (strcmp(fmp->key, "GEOCACHE_PLACER") == 0) {
+	    writebuff(buff, fmp->printfc, NONULL(wpt->gc_data.placer));
+        } else
+	
+	/* GPS STUFF *******************************************************/
+	if (strcmp(fmp->key, "GPS_HDOP") == 0) {
+            writebuff(buff, fmp->printfc, wpt->hdop);
+        } else
+	if (strcmp(fmp->key, "GPS_VDOP") == 0) {
+            writebuff(buff, fmp->printfc, wpt->vdop);
+        } else
+	if (strcmp(fmp->key, "GPS_PDOP") == 0) {
+            writebuff(buff, fmp->printfc, wpt->pdop);
+        } else
+	if (strcmp(fmp->key, "GPS_SAT") == 0) {
+            writebuff(buff, fmp->printfc, wpt->sat);
+        } else
+	if (strcmp(fmp->key, "GPS_FIX") == 0) {
+		char *fix = NULL;
+		switch (wpt->fix) {
+			case fix_unknown:
+				fix = "Unknown";
+				break;
+			case fix_none:
+				fix = "None";
+				break;
+			case fix_2d:
+				fix = "2d";
+				break;
+			case fix_3d:
+				fix = "3d";
+				break;
+			case fix_dgps:
+				fix = "dgps";
+				break;
+			case fix_pps:
+				fix = "pps";
+				break;
+		}
+		writebuff(buff, fmp->printfc, fix);
+        } else
+
+	{
+		/* this should probably never happen */
         }
+	
 
-        fprintf (xcsv_file.xcsvfp, "%s", buff);
-
+        obuff = csv_stringclean(buff, xcsv_file.badchars);
+        fprintf (xcsv_file.xcsvfp, "%s", obuff);
+	xfree(obuff);
     }
 
     fprintf (xcsv_file.xcsvfp, "%s", xcsv_file.record_delimiter);
 
-    if (shortname)
-        xfree(shortname);
-
     if (description && description != shortname)
         xfree(description);
 
+    if (shortname)
+        xfree(shortname);
+
     /* increment the index counter */
     waypt_out_count++;
 }
