@@ -31,9 +31,9 @@ static char *correct_coords = NULL;
 static
 arglist_t dup_args[] = {
 	{"shortname", &snopt, "Suppress duplicate waypoints based on name",
-		NULL, ARGTYPE_BOOL},
+		NULL, ARGTYPE_BEGIN_REQ | ARGTYPE_BOOL},
 	{"location", &lcopt, "Suppress duplicate waypoint based on coords",
-		NULL, ARGTYPE_BOOL},
+		NULL, ARGTYPE_END_REQ | ARGTYPE_BOOL},
 	{"all", &purge_duplicates, "Suppress all instances of duplicates",
 		NULL, ARGTYPE_BOOL},
 	{"correct", &correct_coords, "Use coords from duplicate points", 
