@@ -25,7 +25,7 @@
 static FILE *file_in;
 static FILE *file_out;
 static void *mkshort_handle;
-static char *deficon = NULL;
+/* static char *deficon = NULL; */
 
 #define MYNAME "EasyGPS"
 
@@ -120,6 +120,7 @@ data_read(void)
 				break;
 			case 7:
 				wpt_tmp->icon_descr = (char *) pread();
+				wpt_tmp->wpt_flags.icon_descr_is_dynamic = 1;
 				break;
 			case 8:  /* NULL Terminated (vs. pascal) descr */
 				bbufp = bbuf;
@@ -263,6 +264,7 @@ data_write(void)
 
 ff_vecs_t easygps_vecs = {
 	ff_type_file,
+	FF_CAP_RW_WPT,
 	rd_init,
 	wr_init,
 	rd_deinit,
