@@ -1,7 +1,7 @@
 /*
     Describe vectors containing filter operations.
  
-    Copyright (C) 2002,2004 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002,2004,2005 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@ extern filter_vecs_t routesimple_vecs;
 extern filter_vecs_t reverse_route_vecs;
 extern filter_vecs_t sort_vecs;
 extern filter_vecs_t stackfilt_vecs;
+extern filter_vecs_t trackfilter_vecs;
+extern filter_vecs_t discard_vecs;
 
 static
 fl_vecs_t filter_vec_list[] = {
@@ -85,6 +87,16 @@ fl_vecs_t filter_vec_list[] = {
 		"stack",
 		"Save and restore waypoint lists"
 	},
+	{
+		&trackfilter_vecs,
+		"track",
+		"Manipulate track lists"
+	},
+	{
+		&discard_vecs,
+		"discard",
+		"Remove unreliable points with high hdop or vdop"
+	},
         {
 		NULL,
 		NULL,
@@ -198,6 +210,15 @@ disp_filter_vecs(void)
 	}
 }
 
+static signed int
+alpha (const void *a, const void *b)
+{
+        const fl_vecs_t *const ap = a;
+        const fl_vecs_t *const bp = b;
+
+        return case_ignore_strcmp(ap->desc , bp->desc);
+}
+
 /*
  *  Display the available formats in a format that's easy to machine
  *  parse.   Typically invoked by programs like graphical wrappers to
@@ -208,6 +229,12 @@ disp_filters(int version)
 {
 	fl_vecs_t *vec;
 
+	qsort(filter_vec_list, 
+		sizeof(filter_vec_list) / sizeof(filter_vec_list[0]) - 1,
+		sizeof(filter_vec_list[0]),
+		alpha);
+		
+
 	switch(version) {
 	case 0:
 		for (vec = filter_vec_list; vec->vec; vec++) {
