@@ -43,7 +43,7 @@ arglist_t Args[] = {
 	    "Database name (filename)", NULL, ARGTYPE_STRING },
 	{"category", &Arg_category,
 	    "Category name (Cache)", NULL, ARGTYPE_STRING },
-	{0, 0, 0, 0 }
+	{0, 0, 0, 0, 0 }
 };
 
 #define	ARG_FREE(X) do { if (X) { xfree(X); X = NULL; } } while (0)
@@ -193,7 +193,7 @@ data_read(void)
 
     /* Process record 0 */
     pdb_rec = pdb->rec_index.rec;
-    if (strcmp(pdb_rec->data, Rec0Magic))
+    if (strcmp((char *) pdb_rec->data, Rec0Magic))
 	fatal(MYNAME ": Bad record 0, not a GeoNiche file.\n");
     pdb_rec = pdb_rec->next;
 
@@ -214,9 +214,9 @@ data_read(void)
 	int		icon;
 	char		*notes;
 	char		gid[6+1];
-	struct tm	tm;
+	struct tm	tm  = {0};
 
-	wpt = xcalloc(sizeof(*wpt), 1);
+	wpt = waypt_new();
 	if (!wpt)
 	    fatal(MYNAME ": Couldn't allocate waypoint.\n");
 	vdata = (char *) pdb_rec->data;
@@ -335,7 +335,7 @@ data_read(void)
 	wpt->longitude = lon;
 	wpt->altitude = alt;
 	wpt->icon_descr = category;
-	wpt->icon_descr_is_dynamic = 1;
+	wpt->wpt_flags.icon_descr_is_dynamic = 1;
 
 	if (gid[0])
 	{
@@ -408,7 +408,7 @@ copilot_writewpt(const waypoint *wpt)
     struct pdb_record	*opdb_rec;
     int			vlen;
     static int		vsize = 4096;
-    char		*vdata;
+    ubyte		*vdata;
     char		*title;
     struct tm		tm;
     char		datestr[10+1];
@@ -418,7 +418,7 @@ copilot_writewpt(const waypoint *wpt)
 
     if (ct == 0)
     {
-	opdb_rec = new_Record (0, 0, ct++, sizeof(Rec0Magic), Rec0Magic);	       
+	opdb_rec = new_Record (0, 0, ct++, sizeof(Rec0Magic), (ubyte *) Rec0Magic);	       
 	if (opdb_rec == NULL)
 	    fatal(MYNAME ": libpdb couldn't create record\n");
 	if (pdb_AppendRecord(PdbOut, opdb_rec))
@@ -444,13 +444,13 @@ copilot_writewpt(const waypoint *wpt)
     else
 	notes = enscape(wpt->notes);
 
-    vdata = (char *) xmalloc(vsize);
+    vdata = (ubyte *) xmalloc(vsize);
     if (vdata == NULL)
 	fatal(MYNAME ": libpdb couldn't get record memory\n");
 
     for (;;)
     {
-	vlen = snprintf(vdata, vsize,
+	vlen = snprintf((char *) vdata, vsize,
 	    "Target,%d,%s,,%s,%f,%f,%f,%s,%s,,,,%d,,,,%s"
 	    , id
 	    , title
@@ -479,12 +479,12 @@ copilot_writewpt(const waypoint *wpt)
 	    vsize = vlen + 1;
 	else
 	    vsize *= 2;
-	vdata = (char *) xrealloc(vdata, vsize);
+	vdata = (ubyte *) xrealloc(vdata, vsize);
 	if (vdata == NULL)
 	    fatal(MYNAME ": libpdb couldn't get record memory\n");
     }
 
-    opdb_rec = new_Record (0, 0, ct++, vlen+1, vdata);	       
+    opdb_rec = new_Record (0, 0, ct++, (uword) (vlen+1), vdata);	       
 
     if (opdb_rec == NULL)
 	fatal(MYNAME ": libpdb couldn't create record\n");
@@ -526,6 +526,7 @@ data_write(void)
 ff_vecs_t geoniche_vecs =
 {
 	ff_type_file,
+	FF_CAP_RW_WPT,
 	rd_init,
 	wr_init,
 	rd_deinit,
