@@ -1,7 +1,7 @@
 /*
     Access GPX data files.
 
-    Copyright (C) 2002, 2003, 2004 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002, 2003, 2004, 2005 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  */
 
 #include "defs.h"
+#include "xmlgeneric.h"
 #ifndef NO_EXPAT
 	#include <expat.h>
 	static XML_Parser psr;
@@ -31,6 +32,8 @@ static char *opt_logpoint = NULL;
 static int logpoint_ct = 0;
 
 static const char *gpx_version;
+static char *gpx_wversion;
+static int gpx_wversion_num;
 static const char *gpx_creator;
 static char *xsi_schema_loc;
 
@@ -49,16 +52,27 @@ static int input_string_len = 0;
 
 static time_t file_time;
 
-static char *gsshortnames = NULL;
 static char *snlen = NULL;
 static char *suppresswhite = NULL;
 static char *urlbase = NULL;
 static route_head *trk_head;
 static route_head *rte_head;
 
+
 #define MYNAME "GPX"
 #define MY_CBUF 4096
 #define DEFAULT_XSI_SCHEMA_LOC "http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd"
+#define DEFAULT_XSI_SCHEMA_LOC_FMT "\"http://www.topografix.com/GPX/%c/%c http://www.topografix.com/GPX/%c/%c/gpx.xsd\""
+
+/* 
+ * Format used for floating point formats.  Put in one place to make it
+ * easier to tweak when comparing output with other GPX programs that 
+ * have more or less digits of output...
+ */
+/* #define FLT_FMT "%.9lf" */  /* ExpertGPS */
+#define FLT_FMT "%0.9lf" 
+#define FLT_FMT_T "%0.9lf" 
+#define FLT_FMT_R "%0.9lf" 
 
 typedef enum {
 	tt_unknown = 0,
@@ -77,6 +91,13 @@ typedef enum {
 	tt_wpt_time,
 	tt_wpt_type,
 	tt_wpt_urlname,
+	tt_wpt_link, 		/* New in GPX 1.1 */
+	tt_wpt_link_text, 	/* New in GPX 1.1 */
+	tt_pdop,		/* PDOPS are common for all three */
+	tt_hdop,		/* PDOPS are common for all three */
+	tt_vdop,		/* PDOPS are common for all three */
+	tt_fix,
+	tt_sat,
 	tt_cache,
 	tt_cache_name,
 	tt_cache_container,
@@ -87,6 +108,9 @@ typedef enum {
 	tt_cache_desc_short,
 	tt_cache_desc_long,
 	tt_cache_log_wpt,
+	tt_cache_log_type,
+	tt_cache_log_date,
+	tt_cache_placer,
 	tt_rte,
 	tt_rte_name,
 	tt_rte_desc,
@@ -115,6 +139,8 @@ typedef enum {
 	tt_trk_trkseg_trkpt_desc,
 	tt_trk_trkseg_trkpt_ele,
 	tt_trk_trkseg_trkpt_time,
+	tt_trk_trkseg_trkpt_course,	
+	tt_trk_trkseg_trkpt_speed,
 } tag_type;
 
 typedef struct tag_mapping {
@@ -145,6 +171,8 @@ tag_mapping tag_path_map[] = {
 	{ tt_wpt_desc, 0, "/gpx/wpt/desc" },
 	{ tt_wpt_url, 0, "/gpx/wpt/url" },
 	{ tt_wpt_urlname, 0, "/gpx/wpt/urlname" },
+	{ tt_wpt_link, 0, "/gpx/wpt/link" },			/* GPX 1.1 */
+	{ tt_wpt_link_text, 0, "/gpx/wpt/link/text" },		/* GPX 1.1 */
 	{ tt_wpt_sym, 0, "/gpx/wpt/sym" },
 	{ tt_wpt_type, 1, "/gpx/wpt/type" },
 	{ tt_cache, 1, "/gpx/wpt/groundspeak:cache" },
@@ -157,6 +185,9 @@ tag_mapping tag_path_map[] = {
 	{ tt_cache_desc_short, 1, "/gpx/wpt/groundspeak:cache/groundspeak:short_description" },
 	{ tt_cache_desc_long, 1, "/gpx/wpt/groundspeak:cache/groundspeak:long_description" },
 	{ tt_cache_log_wpt, 1, "/gpx/wpt/groundspeak:cache/groundspeak:logs/groundspeak:log/groundspeak:log_wpt" },
+	{ tt_cache_log_type, 1, "/gpx/wpt/groundspeak:cache/groundspeak:logs/groundspeak:log/groundspeak:type" },
+	{ tt_cache_log_date, 1, "/gpx/wpt/groundspeak:cache/groundspeak:logs/groundspeak:log/groundspeak:date" },
+	{ tt_cache_placer, 1, "/gpx/wpt/groundspeak:cache/groundspeak:owner" },
 
 	{ tt_rte, 0, "/gpx/rte" },
 	{ tt_rte_name, 0, "/gpx/rte/name" },
@@ -186,27 +217,28 @@ tag_mapping tag_path_map[] = {
 	{ tt_trk_trkseg_trkpt_url, 0, "/gpx/trk/trkseg/trkpt/url" },
 	{ tt_trk_trkseg_trkpt_urlname, 0, "/gpx/trk/trkseg/trkpt/urlname" },
 	{ tt_trk_trkseg_trkpt_sym, 0, "/gpx/trk/trkseg/trkpt/sym" },
+	{ tt_trk_trkseg_trkpt_course, 0, "/gpx/trk/trkseg/trkpt/course" },
+	{ tt_trk_trkseg_trkpt_speed, 0, "/gpx/trk/trkseg/trkpt/speed" },
+
+	/* Common to tracks, routes, and waypts */
+	{ tt_fix,  0, "/gpx/wpt/fix" },
+	{ tt_fix,  0, "/gpx/trk/trkseg/trkpt/fix" },
+	{ tt_fix,  0, "/gpx/rte/rtept/fix" },
+	{ tt_sat,  0, "/gpx/wpt/sat" },
+	{ tt_sat,  0, "/gpx/trk/trkseg/trkpt/sat" },
+	{ tt_sat,  0, "/gpx/rte/rtept/sat" },
+	{ tt_pdop, 0, "/gpx/wpt/pdop" },
+	{ tt_pdop, 0, "/gpx/trk/trkseg/trkpt/pdop" },
+	{ tt_pdop, 0, "/gpx/rte/rtept/pdop" },
+	{ tt_hdop, 0, "/gpx/wpt/hdop" },
+	{ tt_hdop, 0, "/gpx/trk/trkseg/trkpt/hdop" },
+	{ tt_hdop, 0, "/gpx/rte/rtept/hdop" },
+	{ tt_vdop, 0, "/gpx/wpt/vdop" },
+	{ tt_vdop, 0, "/gpx/trk/trkseg/trkpt/vdop" },
+	{ tt_vdop, 0, "/gpx/rte/rtept/hdop" },
 	{0}
 };
 
-
-static void
-write_xml_entity(FILE *ofd, const char *indent, 
-                 const char *tag, const char *value)
-{
-	char *tmp_ent = xml_entitize(value);
-	fprintf(ofd, "%s<%s>%s</%s>\n", indent, tag, tmp_ent, tag);
-	xfree(tmp_ent);
-}
-
-static void
-write_optional_xml_entity(FILE *ofd, const char *indent, 
-                          const char *tag, const char *value)
-{
-	if (value && *value)
-		write_xml_entity(ofd, indent, tag, value);
-}
-
 static tag_type
 get_tag(const char *t, int *passthrough)
 {
@@ -298,6 +330,7 @@ start_something_else(const char *el, const char **attrv)
 	char **avcp = NULL;
 	int attr_count = 0;
 	xml_tag *new_tag;
+	fs_xml *fs_gpx = NULL;
        
 	if ( !wpt_tmp ) {
 		return;
@@ -338,8 +371,10 @@ start_something_else(const char *el, const char **attrv)
 		}
 	}
 	else {
-		if ( wpt_tmp->gpx_extras ) {
-			cur_tag = wpt_tmp->gpx_extras;
+		fs_gpx = (fs_xml *)fs_chain_find( wpt_tmp->fs, FS_GPX );
+	       	
+		if ( fs_gpx && fs_gpx->tag ) {
+			cur_tag = fs_gpx->tag;
 			while ( cur_tag->sibling ) {
 				cur_tag = cur_tag->sibling;
 			}
@@ -347,7 +382,9 @@ start_something_else(const char *el, const char **attrv)
 			new_tag->parent = NULL;
 		}
 		else {
-			wpt_tmp->gpx_extras = new_tag;
+			fs_gpx = fs_xml_alloc(FS_GPX);
+			fs_gpx->tag = new_tag;
+			fs_chain_add( &(wpt_tmp->fs), (format_specific_data *)fs_gpx );
 			new_tag->parent = NULL;
 		}
 	}
@@ -427,6 +464,11 @@ gpx_start(void *data, const char *el, const char **attr)
 	case tt_wpt:
 		tag_wpt(attr);
 		break;
+	case tt_wpt_link:
+		if (0 == strcmp(attr[0], "href")) {
+			wpt_tmp->url = xstrdup(attr[1]);
+		}
+		break;
 	case tt_rte:
 		rte_head = route_head_alloc();
 		route_add_head(rte_head);
@@ -468,12 +510,18 @@ gs_type_mapping{
 	geocache_type type;
 	const char *name;
 } gs_type_map[] = {
-	{ gt_traditional, "Traditional cache" },
-	{ gt_multi, "Multi-Cache" },
-	{ gt_virtual, "Virtual cache" },
-	{ gt_event, "Event cache" },
+	{ gt_traditional, "Traditional Cache" },
+	{ gt_multi, "Multi-cache" },
+	{ gt_virtual, "Virtual Cache" },
+	{ gt_event, "Event Cache" },
 	{ gt_webcam, "Webcam Cache" },
-	{ gt_suprise, "Unknown cache" },
+	{ gt_suprise, "Unknown Cache" },
+	{ gt_earth, "Earthcache" },
+	{ gt_cito, "Cache In Trash Out Event" },
+	{ gt_letterbox, "Letterbox Hybrid" },
+	{ gt_locationless, "Locationless (Reverse) Cache" },
+
+	{ gt_benchmark, "Benchmark" }, /* Not Groundspeak; for GSAK  */
 };
 
 struct
@@ -490,7 +538,7 @@ gs_container_mapping{
 };
 
 geocache_type
-gs_mktype(char *t)
+gs_mktype(const char *t)
 {
 	int i;
 	int sz = sizeof(gs_type_map) / sizeof(gs_type_map[0]);
@@ -518,7 +566,7 @@ gs_get_cachetype(geocache_type t)
 }
 
 geocache_container
-gs_mkcont(char *t)
+gs_mkcont(const char *t)
 {
 	int i;
 	int sz = sizeof(gs_container_map) / sizeof(gs_container_map[0]);
@@ -546,7 +594,7 @@ gs_get_container(geocache_container t)
 }
 
 time_t 
-xml_parse_time( char *cdatastr ) 
+xml_parse_time( const char *cdatastr ) 
 {
 	int off_hr = 0;
 	int off_min = 0;
@@ -598,8 +646,7 @@ xml_parse_time( char *cdatastr )
 	tm.tm_year -= 1900;
 	tm.tm_isdst = 0;
 	
-	rv = mktime(&tm) + get_tz_offset() - off_sign*off_hr*3600 - 
-		off_sign*off_min*60;
+	rv = mkgmtime(&tm) - off_sign*off_hr*3600 - off_sign*off_min*60;
 	
         xfree(timestr);
 	
@@ -613,6 +660,7 @@ gpx_end(void *data, const char *el)
 	float x;
 	char *cdatastrp = cdatastr.mem;
 	int passthrough;
+	static time_t gc_log_date;
 
 	if (strcmp(s + 1, el)) {
 		fprintf(stderr, "Mismatched tag %s\n", el);
@@ -643,6 +691,7 @@ gpx_end(void *data, const char *el)
 		wpt_tmp->url = xstrdup(cdatastrp);
 		break;
 	case tt_wpt_urlname:
+	case tt_wpt_link_text:
 		wpt_tmp->url_link_text = xstrdup(cdatastrp);
 		break;
 	case tt_wpt:
@@ -652,11 +701,7 @@ gpx_end(void *data, const char *el)
 		wpt_tmp = NULL;
 		break;
 	case tt_cache_name:
-		if (gsshortnames) {
-			if (wpt_tmp->notes)
-				xfree(wpt_tmp->notes);
-			wpt_tmp->notes = xstrdup(cdatastrp);
-		}
+		wpt_tmp->notes = xstrdup(cdatastrp);
 		break;
 	case tt_cache_container:
 		wpt_tmp->gc_data.container = gs_mkcont(cdatastrp);
@@ -692,6 +737,24 @@ gpx_end(void *data, const char *el)
 		sscanf(cdatastrp, "%f", &x);
 		wpt_tmp->gc_data.terr = x * 10;
 		break;
+	case tt_cache_placer:
+		wpt_tmp->gc_data.placer = xstrdup(cdatastrp);
+		break;
+	case tt_cache_log_date:
+		gc_log_date = xml_parse_time( cdatastrp );
+		break;
+	/*
+	 * "Found it" logs follow the date according to the schema,
+	 * if this is the first "found it" for this waypt, just use the
+	 * last date we saw in this log.
+	 */
+	case tt_cache_log_type:
+		if ((0 == strcmp(cdatastrp, "Found it")) && 
+		    (0 == wpt_tmp->gc_data.last_found)) {
+			wpt_tmp->gc_data.last_found  = gc_log_date;
+		}
+		gc_log_date = 0;
+		break;
 	/*
 	 * Route-specific tags.
  	 */
@@ -726,6 +789,12 @@ gpx_end(void *data, const char *el)
 	case tt_trk_number:
 		trk_head->rte_num = atoi(cdatastrp);
 		break;
+	case tt_trk_trkseg_trkpt_course:
+		wpt_tmp->course = atof(cdatastrp);
+		break;
+	case tt_trk_trkseg_trkpt_speed:
+		wpt_tmp->speed = atof(cdatastrp);
+		break;
 
 	/*
 	 * Items that are actually in multiple categories.
@@ -744,7 +813,7 @@ gpx_end(void *data, const char *el)
 	case tt_rte_rtept_sym:
 	case tt_trk_trkseg_trkpt_sym:
 		wpt_tmp->icon_descr = xstrdup(cdatastrp);
-		wpt_tmp->icon_descr_is_dynamic = 1;
+		wpt_tmp->wpt_flags.icon_descr_is_dynamic = 1;
 		break;
 	case tt_wpt_time:
 	case tt_trk_trkseg_trkpt_time:
@@ -761,6 +830,31 @@ gpx_end(void *data, const char *el)
 	case tt_rte_rtept_desc:
 		wpt_tmp->notes = xstrdup(cdatastrp);
 		break;
+	case tt_pdop:
+		wpt_tmp->pdop = atof(cdatastrp);
+		break;
+	case tt_hdop:
+		wpt_tmp->hdop = atof(cdatastrp);
+		break;
+	case tt_vdop:
+		wpt_tmp->vdop = atof(cdatastrp);
+		break;
+	case tt_sat:
+		wpt_tmp->sat = atof(cdatastrp);
+		break;
+	case tt_fix:
+		wpt_tmp->fix = atoi(cdatastrp)-1;
+		if ( wpt_tmp->fix < fix_2d) {
+			if (!case_ignore_strcmp(cdatastrp, "none"))
+				wpt_tmp->fix = fix_none;
+			else if (!case_ignore_strcmp(cdatastrp, "dgps"))
+				wpt_tmp->fix = fix_dgps;
+			else if (!case_ignore_strcmp(cdatastrp, "pps"))
+				wpt_tmp->fix = fix_pps;
+			else
+				wpt_tmp->fix = fix_unknown;
+		}
+		break;
 	case tt_unknown:
 		end_something_else();
 		*s = 0;
@@ -998,31 +1092,6 @@ gpx_read(void)
 #endif /* NO_EXPAT */
 }
 
-/*
- *
- */
-static
-void
-gpx_write_time(const time_t timep, char *elname)
-{
-	struct tm *tm = gmtime(&timep);
-	
-	if (!tm)
-		return;
-	
-	fprintf(ofd, "<%s>%02d-%02d-%02dT%02d:%02d:%02dZ</%s>\n",
-		elname,
-		tm->tm_year+1900, 
-		tm->tm_mon+1, 
-		tm->tm_mday, 
-		tm->tm_hour, 
-		tm->tm_min, 
-		tm->tm_sec,
-		elname
-	);
-
-}
-
 static void
 fprint_tag_and_attrs( char *prefix, char *suffix, xml_tag *tag )
 {
@@ -1059,7 +1128,7 @@ fprint_xml_chain( xml_tag *tag, const waypoint *wpt )
 			}
 			if ( strcmp(tag->tagname, "groundspeak:cache" ) == 0 
 					&& wpt->gc_data.exported) {
-				gpx_write_time( wpt->gc_data.exported, 
+				xml_write_time( ofd, wpt->gc_data.exported, 
 						"groundspeak:exported" );
 			}
 			fprintf( ofd, "</%s>", tag->tagname);
@@ -1106,12 +1175,106 @@ void free_gpx_extras( xml_tag *tag )
 	}
 }
 
+/*
+ * Handle the grossness of GPX 1.0 vs. 1.1 handling of linky links.
+ */
+static void
+write_gpx_url(const waypoint *waypointp)
+{
+	char *tmp_ent;
+
+	if (waypointp->url) {
+		tmp_ent = xml_entitize(waypointp->url);
+		if (gpx_wversion_num > 10) {
+			
+			fprintf(ofd, "  <link href=\"%s%s\">\n", 
+				urlbase ? urlbase : "", tmp_ent);
+			write_optional_xml_entity(ofd, "  ", "text", 
+				waypointp->url_link_text);
+			fprintf(ofd, "  </link>\n");
+		} else {
+			fprintf(ofd, "  <url>%s%s</url>\n", 
+				urlbase ? urlbase : "", tmp_ent);
+			write_optional_xml_entity(ofd, "  ", "urlname", 
+				waypointp->url_link_text);
+		}
+		xfree(tmp_ent);
+	}
+}
+
+/*
+ * Write optional accuracy information for a given (way|track|route)point
+ * to the output stream.  Done in one place since it's common for all three.
+ * Order counts.
+ */
+static void
+gpx_write_common_acc(const waypoint *waypointp, const char *indent)
+{
+	char *fix = NULL;
+
+	switch (waypointp->fix) {
+		case fix_2d:
+			fix = "2d";
+			break;
+		case fix_3d:
+			fix = "3d";
+			break;
+		case fix_dgps:
+			fix = "dgps";
+			break;
+		case fix_pps:
+			fix = "pps";
+			break;
+	}
+	if (fix) {
+		fprintf(ofd, "%s<fix>%s</fix>\n", indent, fix);
+	}
+	if (waypointp->sat > 0) {
+		fprintf(ofd, "%s<sat>%d</sat>\n", indent, waypointp->sat);
+	}
+	if (waypointp->hdop) {
+		fprintf(ofd, "%s<hdop>%f</hdop>\n", indent, waypointp->hdop);
+	}
+	if (waypointp->vdop) {
+		fprintf(ofd, "%s<vdop>%f</vdop>\n", indent, waypointp->vdop);
+	}
+	if (waypointp->pdop) {
+		fprintf(ofd, "%s<pdop>%f</pdop>\n", indent, waypointp->pdop);
+	}
+}
+
+static void
+gpx_write_common_position(const waypoint *waypointp, const char *indent)
+{
+	if (waypointp->altitude != unknown_alt) {
+		fprintf(ofd, "%s<ele>%f</ele>\n",
+			 indent, waypointp->altitude);
+	}
+	if (waypointp->creation_time) {
+		xml_write_time(ofd, waypointp->creation_time, "time");
+	}
+}
+
+static void
+gpx_write_common_description(const waypoint *waypointp, const char *indent,
+	const char *oname)
+{
+	write_optional_xml_entity(ofd, indent, "name", oname);
+	write_optional_xml_entity(ofd, indent, "cmt", waypointp->description);
+	if (waypointp->notes && waypointp->notes[0])
+		write_xml_entity(ofd, indent, "desc", waypointp->notes);
+	else
+		write_optional_xml_entity(ofd, indent, "desc", waypointp->description);
+	write_gpx_url(waypointp);
+	write_optional_xml_entity(ofd, indent , "sym", waypointp->icon_descr);
+}
+
 static void
 gpx_waypt_pr(const waypoint *waypointp)
 {
-	char *tmp_ent;
 	const char *oname;
 	char *odesc;
+	fs_xml *fs_gpx = NULL;
 
 	/*
 	 * Desparation time, try very hard to get a good shortname
@@ -1128,31 +1291,18 @@ gpx_waypt_pr(const waypoint *waypointp)
 				  mkshort(mkshort_handle, odesc) : 
 				  waypointp->shortname;
 
-	fprintf(ofd, "<wpt lat=\"%0.9lf\" lon=\"%0.9lf\">\n",
+	fprintf(ofd, "<wpt lat=\"" FLT_FMT "\" lon=\"" FLT_FMT "\">\n",
 		waypointp->latitude,
 		waypointp->longitude);
-	if (waypointp->creation_time) {
-		gpx_write_time(waypointp->creation_time, "time");
-	}
-	if (waypointp->altitude != unknown_alt) {
-		fprintf(ofd, "  <ele>%f</ele>\n",
-			 waypointp->altitude);
-	}
-	write_optional_xml_entity(ofd, "  ", "name", oname);
-	write_optional_xml_entity(ofd, "  ", "cmt", waypointp->description);
-	if (waypointp->notes && waypointp->notes[0])
-		write_xml_entity(ofd, "  ", "desc", waypointp->notes);
-	else
-		write_optional_xml_entity(ofd, "  ", "desc", waypointp->description);
-	if (waypointp->url) {
-		tmp_ent = xml_entitize(waypointp->url);
-		fprintf(ofd, "  <url>%s%s</url>\n", urlbase ? urlbase : "", tmp_ent);
-		xfree(tmp_ent);
-	}
-	write_optional_xml_entity(ofd, "  ", "urlname", waypointp->url_link_text);
-	write_optional_xml_entity(ofd, "  ", "sym", waypointp->icon_descr);
 
-	fprint_xml_chain( waypointp->gpx_extras, waypointp );
+	gpx_write_common_position(waypointp, "  ");
+	gpx_write_common_description(waypointp, "  ", oname);
+	gpx_write_common_acc(waypointp, "  ");
+
+	fs_gpx = (fs_xml *)fs_chain_find( waypointp->fs, FS_GPX );
+	if ( fs_gpx ) {
+		fprint_xml_chain( fs_gpx->tag, waypointp );
+	}
 	fprintf(ofd, "</wpt>\n");
 }
 
@@ -1171,16 +1321,32 @@ gpx_track_hdr(const route_head *rte)
 static void
 gpx_track_disp(const waypoint *waypointp)
 {
-	fprintf(ofd, "<trkpt lat=\"%lf\" lon=\"%lf\">\n",
+	fprintf(ofd, "<trkpt lat=\"" FLT_FMT_T "\" lon=\"" FLT_FMT_T "\">\n",
 		waypointp->latitude,
 		waypointp->longitude);
-	if (waypointp->altitude != unknown_alt) {
-		fprintf(ofd, "<ele>%f</ele>\n",
-			 waypointp->altitude);
-	}
-	if (waypointp->creation_time) {
-		gpx_write_time(waypointp->creation_time,"time");
+
+	gpx_write_common_position(waypointp, "  ");
+
+	/* These were accidentally removed from 1.1 */
+	if (gpx_wversion_num == 10) {
+		if (waypointp->course >= 0) {
+			fprintf(ofd, "  <course>%f</course>\n", 
+				waypointp->course);
+		}
+		if (waypointp->speed >= 0) {
+			fprintf(ofd, "  <speed>%f</speed>\n", 
+				waypointp->speed);
+		}
 	}
+
+	/* GPX doesn't require a name on output, so if we made one up
+	 * on input, we might as well say nothing.
+	 */
+	gpx_write_common_description(waypointp, "  ", 
+		waypointp->wpt_flags.shortname_is_synthetic ? 
+			NULL : waypointp->shortname);
+	gpx_write_common_acc(waypointp, "  ");
+
 	fprintf(ofd, "</trkpt>\n");
 }
 
@@ -1211,21 +1377,13 @@ gpx_route_hdr(const route_head *rte)
 static void
 gpx_route_disp(const waypoint *waypointp)
 {
-	fprintf(ofd, "  <rtept lat=\"%f\" lon=\"%f\">\n",
+	fprintf(ofd, "  <rtept lat=\"" FLT_FMT_R "\" lon=\"" FLT_FMT_R "\">\n",
 		waypointp->latitude,
 		waypointp->longitude);
 
-	if (waypointp->altitude != unknown_alt) {
-		fprintf(ofd, "    <ele>%f</ele>\n",
-			 waypointp->altitude);
-	}
-	if (waypointp->creation_time) {
-		gpx_write_time(waypointp->creation_time,"time");
-	}
-	write_optional_xml_entity(ofd, "    ", "name", waypointp->shortname);
-	write_optional_xml_entity(ofd, "    ", "cmt", waypointp->description);
-	write_optional_xml_entity(ofd, "    ", "desc", waypointp->notes);
-	write_optional_xml_entity(ofd, "    ", "sym", waypointp->icon_descr);
+	gpx_write_common_position(waypointp, "    ");
+	gpx_write_common_description(waypointp, "    ", waypointp->shortname);
+	gpx_write_common_acc(waypointp, "    ");
 	fprintf(ofd, "  </rtept>\n");
 }
 
@@ -1248,6 +1406,12 @@ gpx_write(void)
 	time_t now = 0;
 	int short_length;
 	bounds bounds;
+
+	gpx_wversion_num = strtod(gpx_wversion, NULL) * 10;
+
+	if (gpx_wversion_num <= 0) {
+		fatal(MYNAME ": gpx version number of '%s' not valid.\n", gpx_wversion);
+	}
 	
         now = current_time();
 
@@ -1263,13 +1427,23 @@ gpx_write(void)
 	setshort_length(mkshort_handle, short_length);
 
 	fprintf(ofd, "<?xml version=\"1.0\"?>\n");
-	fprintf(ofd, "<gpx\n version=\"1.0\"\n");
+	fprintf(ofd, "<gpx\n version=\"%s\"\n", gpx_wversion);
 	fprintf(ofd, "creator=\"GPSBabel - http://www.gpsbabel.org\"\n");
 	fprintf(ofd, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
-	fprintf(ofd, "xmlns=\"http://www.topografix.com/GPX/1/0\"\n");
-	fprintf(ofd, "xsi:schemaLocation=\"%s\">\n", xsi_schema_loc ? xsi_schema_loc : DEFAULT_XSI_SCHEMA_LOC);
+	fprintf(ofd, "xmlns=\"http://www.topografix.com/GPX/%c/%c\"\n", gpx_wversion[0], gpx_wversion[2]);
+	if (xsi_schema_loc) {
+		fprintf(ofd, "xsi:schemaLocation=\"%s\">\n", xsi_schema_loc);
+	} else {
+		fprintf(ofd,
+			"xsi:schemaLocation=" DEFAULT_XSI_SCHEMA_LOC_FMT">\n",
+			gpx_wversion[0], gpx_wversion[2],
+			gpx_wversion[0], gpx_wversion[2]);
+	}
 
-	gpx_write_time( now, "time" );
+	if (gpx_wversion_num > 10) {	
+		fprintf(ofd, "<metadata>\n");
+	}
+	xml_write_time( ofd, now, "time" );
 	waypt_compute_bounds(&bounds);
 	if (bounds.max_lat  > -360) {
 		fprintf(ofd, "<bounds minlat=\"%0.9f\" minlon =\"%0.9f\" "
@@ -1277,18 +1451,20 @@ gpx_write(void)
 			       bounds.min_lat, bounds.min_lon, 
 			       bounds.max_lat, bounds.max_lon);
 	}
+
+	if (gpx_wversion_num > 10) {	
+		fprintf(ofd, "</metadata>\n");
+	}
+
 	waypt_disp_all(gpx_waypt_pr);
-	gpx_track_pr();
 	gpx_route_pr();
+	gpx_track_pr();
 
 	fprintf(ofd, "</gpx>\n");
 }
 
 static
 arglist_t gpx_args[] = {
-	{ "gsshortnames", &gsshortnames, 
-		"Prefer shorter descriptions from Groundspeak files",
-		NULL, ARGTYPE_BOOL },
 	{ "snlen", &snlen, "Length of generated shortnames", 
 		NULL, ARGTYPE_INT },
 	{ "suppresswhite", &suppresswhite, 
@@ -1299,11 +1475,14 @@ arglist_t gpx_args[] = {
 		NULL, ARGTYPE_BOOL },
 	{ "urlbase", &urlbase, "Base URL for link tag in output", 
 		NULL, ARGTYPE_STRING},
+	{ "gpxver", &gpx_wversion, "Target GPX version for output", 
+		"1.0", ARGTYPE_STRING},
 	{ 0, 0, 0, 0, 0 }
 };
 
 ff_vecs_t gpx_vecs = {
 	ff_type_file,
+	FF_CAP_RW_ALL,
 	gpx_rd_init,	
 	gpx_wr_init,	
 	gpx_rd_deinit,	
