@@ -86,6 +86,7 @@ typedef struct GPS_STrack
     time_t   Time;		/* Unix time */
     float    alt;		/* Altitude */
     float    dpth;		/* Depth    */
+    int32    heartrate;		/* Heatrate as in Garmin 301 */
     int32    tnew;		/* New track? */
     int32    ishdr;		/* Track header? */
     int32    dspl;		/* Display on map? */
@@ -149,6 +150,9 @@ typedef struct GPS_SWay
     int32  rte_link_class;
     char   rte_link_subclass[18];
     char   rte_link_ident[256];
+
+    char     Time_populated;	/* 1 if true */
+    time_t   Time;		/* Unix time */
 } GPS_OWay, *GPS_PWay;
 
 
@@ -179,7 +183,6 @@ extern double gps_save_version;
 extern char   gps_save_string[GPS_ARB_LEN];
 extern int gps_is_usb;
 
-
 extern struct COMMANDDATA COMMAND_ID[2];
 extern struct LINKDATA LINK_ID[3];
 extern struct GPS_MODEL_PROTOCOL GPS_MP[];
@@ -190,7 +193,6 @@ extern char *gps_aviation_sym[];
 extern char *gps_16_sym[];
 
 
-
 #endif
 
 #ifdef __cplusplus
