@@ -141,6 +141,7 @@ int32 gps_pvt_transfer;
 #define pD107 107
 #define pD108 108
 #define pD109 109
+#define pD110 110
 #define pD150 150
 #define pD151 151
 #define pD152 152
@@ -173,6 +174,7 @@ int32 gps_rte_link_type;
 #define pD300 300
 #define pD301 301
 #define pD302 302
+#define pD303 303
 int32 gps_trk_type;
 
 
@@ -246,7 +248,7 @@ int32 gps_link_type;
 
 struct GPS_MODEL_PROTOCOL
 {
-    int32 id;
+    US    id;
     int32 link;
     int32 command;
     int32 wayptt;
