@@ -83,19 +83,20 @@ gusb_open(const char *pname)
 // }
 	}
 
-	hdevinfo = SetupDiGetClassDevs( &GARMIN_GUID, NULL, NULL, 
+	hdevinfo = SetupDiGetClassDevs( (GUID *) &GARMIN_GUID, NULL, NULL, 
 			DIGCF_PRESENT|DIGCF_INTERFACEDEVICE);
 
 	if (hdevinfo == INVALID_HANDLE_VALUE) {
-		fatal("XXX");
+		GPS_Serial_Error("SetupDiGetClassDevs failed");
 		return 0;
 	}
 
 	/* Get the device associated with this index. */
 	devinterface.cbSize = sizeof(SP_INTERFACE_DEVICE_DATA);
-	if (!SetupDiEnumDeviceInterfaces(hdevinfo, NULL, &GARMIN_GUID, 
+	if (!SetupDiEnumDeviceInterfaces(hdevinfo, NULL, (GUID *) &GARMIN_GUID, 
 			0, &devinterface)) {
-		fatal("blah");
+		GPS_Serial_Error("SetupDiEnumDeviceInterfaces");
+		warning("Is the unit powered up and connected?");
 		return 0;
 	}
 
@@ -107,7 +108,8 @@ gusb_open(const char *pname)
 	devinfo.cbSize = sizeof(SP_DEVINFO_DATA);
 
 	if (!SetupDiGetDeviceInterfaceDetail(hdevinfo, &devinterface, pdd, size, NULL, &devinfo)) {
-		fatal("ZZZ");
+		GPS_Serial_Error("SetupDiGetDeviceInterfaceDetail");
+		return 0;
 	}
 
 	/* Whew.  All that just to get something we can open... */
@@ -117,12 +119,12 @@ gusb_open(const char *pname)
 			0, NULL, OPEN_EXISTING, 0, NULL );
 	if (usb_handle == INVALID_HANDLE_VALUE) {
 		GPS_Serial_Error("CreateFile failed");
-		fatal("CreateFile");
+		return 0;
 	}
 
 	if(!DeviceIoControl(usb_handle, IOCTL_GARMIN_USB_BULK_OUT_PACKET_SIZE, NULL, 0,
 			&usb_tx_packet_size, GARMIN_USB_INTERRUPT_DATA_SIZE, &size, NULL)) {
-		fatal("Couldn't get USB packet size");
+		fatal("Couldn't get USB packet size.\n");
 	}
 
 	if (pdd) {
@@ -171,13 +173,14 @@ gusb_close(const char *portname)
 		usb_handle = INVALID_HANDLE_VALUE;
 #endif
 	}
+  return 0;
 }
 
 int
 gusb_cmd_get(garmin_usb_packet *ibuf, size_t sz)
 {
 	DWORD rxed = GARMIN_USB_INTERRUPT_DATA_SIZE;
-	unsigned char *buf = &ibuf->dbuf;
+	unsigned char *buf = (unsigned char *) &ibuf->dbuf;
 	int i;
 	int tsz=0;
 	unsigned char *obuf = buf;
@@ -189,10 +192,10 @@ gusb_cmd_get(garmin_usb_packet *ibuf, size_t sz)
 	if(!DeviceIoControl(usb_handle, IOCTL_GARMIN_USB_INTERRUPT_IN, NULL, 0,
 			buf, GARMIN_USB_INTERRUPT_DATA_SIZE, &rxed, NULL)) {
 		GPS_Serial_Error("Ioctl");
-		fatal("ioctl");
+		fatal("ioctl\n");
 	}
 		buf += rxed;
-		sz =- rxed;
+		sz  -= rxed;
 		tsz += rxed;
 		if (rxed < GARMIN_USB_INTERRUPT_DATA_SIZE) {
 			break;
@@ -219,7 +222,7 @@ gusb_cmd_send(const garmin_usb_packet *opkt, size_t sz)
 {
 	DWORD rsz;
 	size_t i;
-	unsigned char *obuf = &opkt->dbuf;
+	unsigned char *obuf = (unsigned char *) &opkt->dbuf;
 	const char *m1, *m2;
 
 	/* The spec warns us about making writes an exact multiple
@@ -253,7 +256,7 @@ gusb_cmd_send(const garmin_usb_packet *opkt, size_t sz)
 static char *
 id_unit(void)
 {
-static const char  oid[12] = {20, 0, 0, 0, 0xfe, 0, 0, 0, 0, 0, 0, 0};
+static const unsigned char  oid[12] = {20, 0, 0, 0, 0xfe, 0, 0, 0, 0, 0, 0, 0};
 	/* 
 	 * Identify the unit before getting into all the protocol gunk.
 	 * We get two packets back, but we discard the protocol array 
