@@ -1,7 +1,7 @@
 /*
     Communicate Thales/Magellan serial protocol.
 
-    Copyright (C) 2002, 2003, 2004 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002, 2003, 2004, 2005 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,22 +26,27 @@
 #include "magellan.h"
 
 int bitrate = 4800;
+int wptcmtcnt;
+int wptcmtcnt_max;
 #define MYNAME "MAGPROTO"
+#define MAXCMTCT 200
 
 #define debug_serial  (global_opts.debug_level > 1)
 
 static char * termread(char *ibuf, int size);
 static void termwrite(char *obuf, int size);
-static void mag_readmsg(void);
+static void mag_readmsg(gpsdata_type objective);
 static void mag_handon(void);
 static void mag_handoff(void);
 static void *mkshort_handle = NULL;
 static char *deficon = NULL;
 static char *bs = NULL;
+static char *cmts = NULL;
 static char *noack = NULL;
 static char *nukewpt = NULL;
 static int route_out_count;
 static int waypoint_read_count;
+static int wpt_len = 8;
 
 typedef enum {
 	mrs_handoff = 0,
@@ -171,7 +176,9 @@ pid_to_model_t pid_to_model[] =
 	{ mm_meridian, 35, "ProMark 2" },
 	{ mm_sportrak, 36, "SporTrak Map/Pro" },
 	{ mm_sportrak, 37, "SporTrak" },
+	{ mm_meridian, 38, "FX324 Plotter" },
 	{ mm_meridian, 39, "Meridian Color" },
+	{ mm_meridian, 40, "FX324C Plotter" },
 	{ mm_sportrak, 41, "Sportrak Color" },
 	{ mm_sportrak, 42, "Sportrak Marine" },
 	{ mm_meridian, 43, "Meridian Marine" },
@@ -207,7 +214,7 @@ m315_cleanse(char *istring)
 /*
  * Do same for 330, Meridian, and SportTrak.
  */
-static char *
+char *
 m330_cleanse(char *istring)
 {
 	static char m330_valid_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ "
@@ -229,7 +236,7 @@ m330_cleanse(char *istring)
  * Given a protocol message, compute the checksum as needed by 
  * the Magellan protocol.
  */
-static unsigned int 
+unsigned int 
 mag_checksum(const char * const buf)
 {
 	int csum = 0;
@@ -270,7 +277,7 @@ mag_writemsg(const char * const buf)
 	termwrite(obuf, i);
 	if (magrxstate == mrs_handon || magrxstate == mrs_awaiting_ack) {
 		magrxstate = mrs_awaiting_ack;
-		mag_readmsg();
+		mag_readmsg(trkdata);
 		if (last_rx_csum != osum) {
 			if (debug_serial) {
 				warning("COMM ERROR: Expected %02x, got %02x", 
@@ -359,7 +366,7 @@ mag_verparse(char *ibuf)
 		case mm_meridian:
 		case mm_sportrak:
 			icon_mapping = map330_icon_table;
-			setshort_length(mkshort_handle, 8);
+			setshort_length(mkshort_handle, wpt_len);
 			setshort_mustupper(mkshort_handle, 0);
 			mag_cleanse = m330_cleanse;
 			break;
@@ -371,7 +378,7 @@ mag_verparse(char *ibuf)
 #define IS_TKN(x) (strncmp(ibuf,x, sizeof(x)-1) == 0)
 
 static void
-mag_readmsg(void)
+mag_readmsg(gpsdata_type objective)
 {
 	char ibuf[200];
 	int isz;
@@ -438,7 +445,7 @@ retry:
 			waypt_status_disp(waypoint_read_count, 
 					waypoint_read_count);
 		}
-		switch (global_opts.objective)
+		switch (objective)
 		{
 			case wptdata:
 				waypt_add(wpt);
@@ -611,7 +618,7 @@ termwrite(char *obuf, int size)
 		return;
 	}
 	WriteFile (comport, obuf, size, &len, NULL);
-	if (len != size) {
+	if ((int) len != size) {
 		fatal(MYNAME ":.  Wrote %d of %d bytes.\n", len, size);
 	}
 }
@@ -711,6 +718,8 @@ static
 arglist_t mag_sargs[] = {
 	{"baud", &bs, "Numeric value of bitrate (baud=4800)", NULL,
 		ARGTYPE_INT },
+	{"maxcmts", &cmts, "Max number of comments to write (maxcmts=200)", 
+		NULL, ARGTYPE_INT },
 	{"noack", &noack, "Suppress use of handshaking in name of speed",
 		NULL, ARGTYPE_BOOL},
 	{"deficon", &deficon, "Default icon name", NULL, ARGTYPE_STRING },
@@ -754,7 +763,7 @@ mag_rd_init(const char *portname)
 	}
 
 	while (!got_version) {
-		mag_readmsg();
+		mag_readmsg(trkdata);
 		if (current_time() > later) {
 			fatal(MYNAME ": No acknowledgment from GPS on %s\n",
 				portname);
@@ -773,7 +782,7 @@ mag_rd_init(const char *portname)
 	if (nukewpt) {
 		/* The unit will send us an "end" message upon completion */
 		mag_writemsg("PMGNCMD,DELETE,WAYPOINT");
-		mag_readmsg();
+		mag_readmsg(trkdata);
 		if (!found_done) {
 			fatal(MYNAME ": Unexpected response to waypoint delete command.\n");
 		}
@@ -786,8 +795,18 @@ mag_rd_init(const char *portname)
 }
 
 static void
-mag_wr_init(const char *portname)
+mag_wr_init_common(const char *portname)
 {
+	if (bs) {
+		bitrate=atoi(bs);
+	}
+
+	if (cmts) {
+		wptcmtcnt_max = atoi(cmts);
+	} else {
+		wptcmtcnt_max = MAXCMTCT ;
+	}
+
 #if __WIN32__
 	if (!terminit(portname, 1)) {
 		is_file = 1;
@@ -823,6 +842,24 @@ mag_wr_init(const char *portname)
 	QUEUE_INIT(&rte_wpt_tmp);
 }
 
+/*
+ * Entry point for extended (explorist) points.
+ */
+static void
+magX_wr_init(const char *portname)
+{
+	wpt_len = 20;
+	mag_wr_init_common(portname);
+	setshort_length(mkshort_handle, wpt_len);
+}
+
+static void
+mag_wr_init(const char *portname)
+{
+	wpt_len = 8;
+	mag_wr_init_common(portname);
+}
+
 static void
 mag_deinit(void)
 {
@@ -857,7 +894,7 @@ mag_trkparse(char *trkmsg)
 	struct tm tm;
 	waypoint *waypt;
 
-	waypt  = xcalloc(sizeof *waypt, 1);
+	waypt  = waypt_new();
 
 	memset(&tm, 0, sizeof(tm));
 
@@ -878,7 +915,7 @@ mag_trkparse(char *trkmsg)
 	dmy = dmy / 100;
 	tm.tm_mday = dmy % 100; 
 
-	waypt->creation_time = mktime(&tm) + get_tz_offset();
+	waypt->creation_time = mkgmtime(&tm);
 	waypt->centiseconds = fracsecs;
 
 	if (latdir == 'S') latdeg = -latdeg;
@@ -1052,7 +1089,7 @@ mag_wptparse(char *trkmsg)
 	descr[0] = 0;
 	icon_token[0] = 0;
 
-	waypt  = xcalloc(sizeof *waypt, 1);
+	waypt  = waypt_new();
 
 	sscanf(trkmsg,"$PMGNWPL,%lf,%c,%lf,%c,%d,%c,%[^,],%[^,]", 
 		&latdeg,&latdir,
@@ -1084,58 +1121,52 @@ mag_read(void)
 {
 	found_done = 0;
 
-	switch (global_opts.objective)
-	{
-		case trkdata:
-			if (!is_file) 
-				mag_writemsg("PMGNCMD,TRACK,2");
+        if (global_opts.masked_objective & TRKDATAMASK) {
+          if (!is_file) 
+            mag_writemsg("PMGNCMD,TRACK,2");
+          
+          while (!found_done) {
+            mag_readmsg(trkdata);
+          }
+        }
 
-			while (!found_done) {
-				mag_readmsg();
-			}
+        if (global_opts.masked_objective & WPTDATAMASK) {
+          if (!is_file) 
+            mag_writemsg("PMGNCMD,WAYPOINT");
+          
+          while (!found_done) {
+            mag_readmsg(wptdata);
+          }
+        }
 
-			break;
-		case wptdata:
-			if (!is_file) 
-				mag_writemsg("PMGNCMD,WAYPOINT");
-
-			while (!found_done) {
-				mag_readmsg();
-			}
-
-			break;
-		case rtedata:
-			if (!is_file) {
-				/* 
-				 * serial routes require waypoint & routes 
-				 * messages commands.
-				 */
-				mag_writemsg("PMGNCMD,WAYPOINT");
-
-				while (!found_done) {
-					mag_readmsg();
-				}
-
-				mag_writemsg("PMGNCMD,ROUTE");
-
-				found_done = 0;
-				while (!found_done) {
-					mag_readmsg();
-				}
-			} else {
-				/*
-				 * SD routes are a stream of PMGNWPL and 
-				 * PMGNRTE messages, in that order.
-				 */
-				while (!found_done) {
-					mag_readmsg();
-				}
-			}
-
-			break;
-		default:
-			fatal(MYNAME ": Unknown objective\n");
-	}
+        if (global_opts.masked_objective & RTEDATAMASK) {
+          if (!is_file) {
+            /* 
+             * serial routes require waypoint & routes 
+             * messages commands.
+             */
+            mag_writemsg("PMGNCMD,WAYPOINT");
+            
+            while (!found_done) {
+              mag_readmsg(rtedata);
+            }
+            
+            mag_writemsg("PMGNCMD,ROUTE");
+            
+            found_done = 0;
+            while (!found_done) {
+              mag_readmsg(rtedata);
+            }
+          } else {
+            /*
+             * SD routes are a stream of PMGNWPL and 
+             * PMGNRTE messages, in that order.
+             */
+            while (!found_done) {
+              mag_readmsg(rtedata);
+            }
+          }
+        }          
 }
 
 static
@@ -1150,7 +1181,7 @@ mag_waypt_pr(const waypoint *waypointp)
 	const char *icon_token=NULL;
 	char *owpt;
 	char *odesc;
-	char *isrc;
+	char *isrc = NULL;
 
 	ilat = waypointp->latitude;
 	ilon = waypointp->longitude;
@@ -1179,11 +1210,12 @@ mag_waypt_pr(const waypoint *waypointp)
 
 	isrc = waypointp->notes ? waypointp->notes : waypointp->description;
 	owpt = global_opts.synthesize_shortnames ?
-                        mkshort(mkshort_handle, isrc) : waypointp->shortname;
+                        mkshort_from_wpt(mkshort_handle, waypointp) : waypointp->shortname;
 	odesc = isrc ? isrc : "";
 	owpt = mag_cleanse(owpt);
 
-	if (waypointp->gc_data.diff && waypointp->gc_data.terr) {
+	if (!global_opts.no_smart_icons &&
+	    waypointp->gc_data.diff && waypointp->gc_data.terr) {
 		sprintf(ofmtdesc, "%d/%d %s", waypointp->gc_data.diff, 
 			waypointp->gc_data.terr, odesc);
 		odesc = mag_cleanse(ofmtdesc);
@@ -1191,11 +1223,20 @@ mag_waypt_pr(const waypoint *waypointp)
 		odesc = mag_cleanse(odesc);
 	}
 
-	sprintf(obuf, "PMGNWPL,%4.3f,%c,%09.3f,%c,%07.lf,M,%-.8s,%-.30s,%s",
+	/*
+	 * For the benefit of DirectRoute (which uses waypoint comments
+	 * to deliver turn-by-turn popups for street routing) allow a 
+	 * cap on the comments delivered so we leave space for it to route.
+	 */
+	if (odesc && /* !is_file && */ (wptcmtcnt++ >= wptcmtcnt_max))
+		odesc[0] = 0;
+
+	sprintf(obuf, "PMGNWPL,%4.3f,%c,%09.3f,%c,%07.lf,M,%-.*s,%-.46s,%s",
 		lat, ilat < 0 ? 'S' : 'N',
 		lon, ilon < 0 ? 'W' : 'E',
 		waypointp->altitude == unknown_alt ?
 			0 : waypointp->altitude,
+		wpt_len,
 		owpt,
 		odesc,
 		icon_token);
@@ -1332,7 +1373,8 @@ mag_route_trl(const route_head * rte)
 			thisline++;
 
 			sprintf(obuff, "PMGNRTE,%d,%d,c,%d,%s,%s", 
-				numlines, thisline, route_out_count,
+				numlines, thisline, 
+				rte->rte_num ? rte->rte_num : route_out_count,
 				buff1, buff2);
 
 			mag_writemsg(obuff);
@@ -1363,7 +1405,11 @@ mag_write(void)
 	 * Whitespace is actually legal, but since waypoint name length is
 	 * only 8 bytes, we'll conserve them.
 	 */
+
 	setshort_whitespace_ok(mkshort_handle, 0);
+
+	wptcmtcnt = 0;
+
 	switch (global_opts.objective)
 	{
 		case trkdata:
@@ -1386,6 +1432,7 @@ mag_write(void)
  */
 ff_vecs_t mag_svecs = {
 	ff_type_serial,
+	FF_CAP_RW_ALL,
 	mag_rd_init,	
 	mag_wr_init,	
 	mag_deinit,	
@@ -1398,6 +1445,7 @@ ff_vecs_t mag_svecs = {
 
 ff_vecs_t mag_fvecs = {
 	ff_type_file,
+	FF_CAP_RW_ALL,
 	mag_rd_init,	
 	mag_wr_init,	
 	mag_deinit,	
@@ -1407,3 +1455,18 @@ ff_vecs_t mag_fvecs = {
 	NULL, 
 	mag_fargs
 };
+
+/*
+ * Extended (Explorist) entry tables.
+ */
+ff_vecs_t magX_fvecs = {
+	ff_type_file,
+	FF_CAP_RW_ALL,
+	mag_rd_init,	
+	magX_wr_init,	
+	mag_deinit,	
+	mag_deinit,	
+	mag_read,
+	mag_write,
+	NULL, 
+};
