@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002-2005 Robert Lipe, robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ usage(const char *pname, int shorter)
 "    The input type and filename are specified with the -i INTYPE\n"
 "    and -f INFILE options. The output type and filename are specified\n"
 "    with the -o OUTTYPE and -F OUTFILE options.\n"
+"    If '-' is used for INFILE or OUTFILE, stdin or stdout will be used.\n"
 "\n"
 "    In the second form of the command, INFILE and OUTFILE are the\n"
 "    first and second positional (non-option) arguments.\n"
@@ -53,7 +54,7 @@ usage(const char *pname, int shorter)
 "    -t               Process track information\n"
 "    -w               Process waypoint information [default]\n"
 "    -N               No smart icons on output\n"
-"    -x filtername    Invoke filter\n"
+"    -x filtername    Invoke filter (place between inputs and output) \n"
 "    -D level         Set debug level [%d]\n"
 "    -h, -?           Print detailed help and exit\n"
 "    -V               Print GPSBabel version and exit\n"
@@ -152,6 +153,9 @@ main(int argc, char *argv[])
 				optarg = argv[argn][2]
 					? argv[argn]+2 : argv[++argn];
 				ovecs = find_vec(optarg, &ovec_opts);
+				if (ovecs == NULL) {
+					fatal ("Output type '%s' not recognized\n", optarg);
+				}
 				break;
 			case 'f':
 				optarg = argv[argn][2]
@@ -202,7 +206,19 @@ main(int argc, char *argv[])
 				global_opts.masked_objective |= RTEDATAMASK;
 				break;
 			case 'N':
-				global_opts.no_smart_icons = 1;
+				switch(argv[argn][2]) {
+					case 'i':
+						global_opts.no_smart_icons = 1;
+						break;
+					case 'n':
+						global_opts.no_smart_names = 1;
+						break;
+					default:
+						global_opts.no_smart_names = 1;
+						global_opts.no_smart_icons = 1;
+						break;
+				}
+				
 				break;
  			case 'x':
 				optarg = argv[argn][2]
@@ -236,6 +252,7 @@ main(int argc, char *argv[])
 			case 'v':
 				switch(argv[argn][2]) {
 				case 's': global_opts.verbose_status = 1; break;
+				case 'S': global_opts.verbose_status = 2; break;
 				}
 				break;
 
