@@ -256,7 +256,8 @@ static void write_header( void ) {
 		--recs;
 	}
 	
-	opdb_rec = new_Record (0, 0, 0, sizeof(struct doc_record0)+sizeof(short)*(ct-1), (const ubyte *)rec0);
+	opdb_rec = new_Record (0, 0, 0, 
+		(uword) (sizeof(struct doc_record0)+sizeof(short)*(ct-1)), (const ubyte *)rec0);
 
 	if (opdb_rec == NULL) {
 		fatal(MYNAME ": libpdb couldn't create summary record\n");
@@ -265,6 +266,7 @@ static void write_header( void ) {
 	if (pdb_InsertRecord(opdb, NULL, opdb_rec)) {
 		fatal(MYNAME ": libpdb couldn't insert summary record\n");
 	}
+	xfree(rec0);
 }
 
 static void write_bookmarks( void ) {
@@ -288,7 +290,7 @@ static void write_bookmarks( void ) {
 		bookmark_tail = oldmark->next;
 		
 		be_write32( &rec.offset, oldmark->offset );
-		memset( rec.text, 16, 0 );
+		memset( rec.text, 0, 16 );
 		strncpy( rec.text, oldmark->text, 16 );
 		
 		opdb_rec = new_Record( 0, 0, ct++, 
@@ -302,7 +304,6 @@ static void write_bookmarks( void ) {
 			fatal(MYNAME ": libpdb couldn't append bookmark record\n");
 		}
 
-		
 		xfree( oldmark );
 	} 
 }
@@ -316,7 +317,7 @@ static void commit_buffer( void ) {
 
 	compress( &buf );
 	
-        opdb_rec = new_Record (0, 0, ct++, buf.len, (const ubyte *)buf.data);
+        opdb_rec = new_Record (0, 0, ct++, (uword) buf.len, (const ubyte *)buf.data);
 
         if (opdb_rec == NULL) {
                 fatal(MYNAME ": libpdb couldn't create record\n");
@@ -356,7 +357,7 @@ static void docprintf( int maxlen, char *format, ... ) {
 	partlen = BUFFER_SIZE-1-buf.len;	
     	if ( buf.len + newlen + 1 > BUFFER_SIZE ) 
     	{
-	    strncpy( buf.data+buf.len, txt2, partlen );
+	    strncpy( (char *) buf.data+buf.len, txt2, partlen );
             buf.data[BUFFER_SIZE-1] = '\0';
             txt2 += partlen;
             newlen -= partlen;
@@ -365,7 +366,7 @@ static void docprintf( int maxlen, char *format, ... ) {
             NEW_BUFFER( &buf );
         }
         else { 
-            strcpy( buf.data+buf.len, txt2 );
+            strcpy( (char *) buf.data+buf.len, txt2 );
             buf.len += newlen;
             txt2 = NULL;
         }
@@ -414,26 +415,31 @@ palmdoc_disp(const waypoint *wpt)
 	int latint, lonint;
 	char tbuf[1024];
 	time_t tm = wpt->creation_time;
-	long utmz;
+	int32 utmz;
 	double utme, utmn;
 	char utmzc;
+	char *bm;
+	fs_xml *fs_gpx = NULL;
 
         char bookmarktext[17];
 
         if ( bmid ) {
+		char * s = mkshort_from_wpt(mkshort_bookmark_handle, wpt);
 		sprintf( bookmarktext, "%6s:%9s", 
-			wpt->shortname?wpt->shortname:"",
-			mkshort(mkshort_bookmark_handle, wpt->description));
+			wpt->shortname?wpt->shortname:"",s);
+		xfree(s);
 	}
 	else {
-		sprintf( bookmarktext, "%16s", 
-			mkshort(mkshort_bookmark_handle, wpt->description));
+		char * s = mkshort_from_wpt(mkshort_bookmark_handle, wpt);
+		sprintf( bookmarktext, "%16s", s);
+		xfree(s);
 	}	
-	
-        create_bookmark(xstrdup(bookmarktext)); 
+
+        bm = xstrdup(bookmarktext); 
+        create_bookmark(bm);
  	
-	lonint = abs(wpt->longitude);
-	latint = abs(wpt->latitude);
+	lonint = abs((int) wpt->longitude);
+	latint = abs((int) wpt->latitude);
 
 	GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude, 
 		&utme, &utmn, &utmz, &utmzc);
@@ -443,7 +449,7 @@ palmdoc_disp(const waypoint *wpt)
 	strftime(tbuf, sizeof(tbuf), "%d-%b-%Y", localtime(&tm));
 
 	docprintf(300, "%-16s  %c%d %06.3f  %c%d %06.3f  (%ld%c %6.0f %7.0f)",
-		(global_opts.synthesize_shortnames) ? mkshort(mkshort_handle, wpt->description) : wpt->shortname,
+		(global_opts.synthesize_shortnames) ? mkshort_from_wpt(mkshort_handle, wpt) : wpt->shortname,
 		wpt->latitude < 0 ? 'S' : 'N',  abs(latint), 60.0 * (fabs(wpt->latitude) - latint), 
 		wpt->longitude < 0 ? 'W' : 'E', abs(lonint), 60.0 * (fabs(wpt->longitude) - lonint),
 		utmz, utmzc, utme, utmn);
@@ -477,9 +483,14 @@ palmdoc_disp(const waypoint *wpt)
 	else if (wpt->notes && (!wpt->description || strcmp(wpt->notes,wpt->description))) {
 		docprintf (10+strlen(wpt->notes), "%s\n", wpt->notes);
 	}
-
-	if ( includelogs && wpt->gpx_extras ) {
-		xml_tag *root = wpt->gpx_extras;
+        
+	fs_gpx = NULL;
+        if ( includelogs ) {
+	        fs_gpx = (fs_xml *)fs_chain_find( wpt->fs, FS_GPX);
+	}
+		
+        if ( fs_gpx && fs_gpx->tag ) {
+                xml_tag *root = fs_gpx->tag;
 		xml_tag *curlog = NULL;
 		xml_tag *logpart = NULL;
 		curlog = xml_findfirst( root, "groundspeak:log" );
@@ -604,6 +615,7 @@ data_write(void)
 
 ff_vecs_t palmdoc_vecs = {
 	ff_type_file,
+	{ ff_cap_write, ff_cap_none, ff_cap_none},
 	NULL,
 	wr_init,
 	NULL,
