@@ -62,6 +62,7 @@ arglist_t psit_args[] = {
 };
 
 /* Taken from PsiTrex 1.13 */
+static
 const psit_icon_mapping_t psit_icon_value_table[] = {
 	{   0x00, "anchor" },
 	{   0x06, "dollar" },
@@ -144,7 +145,7 @@ const psit_icon_mapping_t psit_icon_value_table[] = {
 	{     -1, NULL }
 };
 
-const char *
+static const char *
 psit_find_desc_from_icon_number(const int icon)
 {
 	const psit_icon_mapping_t *i;
@@ -157,7 +158,7 @@ psit_find_desc_from_icon_number(const int icon)
 	return "";
 }
 
-int
+static int
 psit_find_icon_number_from_desc(const char *desc)
 {
 	const psit_icon_mapping_t *i;
@@ -208,7 +209,6 @@ static void
 psit_getToken(FILE *psit_file, char *buf, size_t sz, psit_tokenSep_type delimType)
 {
 	int c;
-char *buf2 = buf;	/* MRCB debug */
 
 	*buf = 0;
 
@@ -285,8 +285,6 @@ psit_waypoint_r(FILE *psit_file, waypoint **wpt)
 	int		garmin_icon_num;
 
 	waypoint	*thisWaypoint;
-	double	psit_altitude = unknown_alt;
-	double	psit_depth = unknown_alt;
 
 	if (strlen(psit_current_token) > 0) {
 		thisWaypoint = waypt_new();
@@ -331,10 +329,8 @@ static void
 psit_waypoint_w(FILE *psit_file, const waypoint *wpt)
 {
 	int	icon;
-	char *src;
-	const char *ident;
-	int display = 1;
-	int colour = 0;			/*  (unknown colour) black is 1, white is 16 */
+    const char *ident;
+	char *src = 0;  /* BUGBUG Passed to mkshort */
 
 	fprintf(psit_file, "%11.6f,%11.6f,", 
 						wpt->latitude,
@@ -383,13 +379,10 @@ psit_route_r(FILE *psit_file, route_head **rte)
 
 	int		garmin_icon_num;
 
-	time_t	dateTime = 0;
 	route_head *rte_head;
 	unsigned int rte_count;
 
 	waypoint	*thisWaypoint;
-	double	psit_altitude = unknown_alt;
-	double	psit_depth = unknown_alt;
 
 	psit_getToken(psit_file,psit_current_token,sizeof(psit_current_token), ltrimEOL);
 
@@ -525,8 +518,6 @@ psit_track_r(FILE *psit_file, route_head **trk)
 	unsigned int trk_count;
 
 	waypoint	*thisWaypoint;
-	double	psit_altitude = unknown_alt;
-	double	psit_depth = unknown_alt;
 
 	psit_getToken(psit_file,psit_current_token,sizeof(psit_current_token), ltrimEOL);
 	if (strlen(psit_current_token) == 0) {
@@ -580,7 +571,7 @@ psit_track_r(FILE *psit_file, route_head **trk)
 					&(tmTime.tm_sec));
 
 			tmTime.tm_isdst = 0;
-			dateTime = mktime(&tmTime) + get_tz_offset();
+			dateTime = mkgmtime(&tmTime);
 
 			psit_getToken(psit_file,psit_current_token,sizeof(psit_current_token), whitespace);
 
@@ -676,10 +667,6 @@ psit_trackdatapoint_w(FILE *psit_file, const waypoint *wpt)
 	time_t	t = wpt->creation_time;
 	struct tm *tmTime = gmtime(&t);
 
-	double	psit_altitude = wpt->altitude;
-	double	psit_proximity = unknown_alt;
-	double	psit_depth = unknown_alt;
-
 	fprintf(psit_file, "%11.6f,%11.6f,", 
 						wpt->latitude,
 						wpt->longitude);
@@ -799,6 +786,7 @@ psit_write(void)
 
 ff_vecs_t psit_vecs = {
 	ff_type_file,
+	FF_CAP_RW_ALL,
 	psit_rd_init,
 	psit_wr_init,
 	psit_rd_deinit,
