@@ -67,12 +67,13 @@ text_disp(const waypoint *wpt)
 	int latint, lonint;
 	char tbuf[1024];
 	time_t tm = wpt->creation_time;
-	long utmz;
+	int32 utmz;
 	double utme, utmn;
 	char utmzc;
+	fs_xml *fs_gpx;
 	
-	lonint = abs(wpt->longitude);
-	latint = abs(wpt->latitude);
+	lonint = abs((int) wpt->longitude);
+	latint = abs((int) wpt->latitude);
 
 	GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude, 
 		&utme, &utmn, &utmz, &utmzc);
@@ -82,7 +83,7 @@ text_disp(const waypoint *wpt)
 	strftime(tbuf, sizeof(tbuf), "%d-%b-%Y", localtime(&tm));
 
 	fprintf(file_out, "%-16s  %c%d %06.3f  %c%d %06.3f  (%ld%c %6.0f %7.0f)",
-		(global_opts.synthesize_shortnames) ? mkshort(mkshort_handle, wpt->description) : wpt->shortname,
+		(global_opts.synthesize_shortnames) ? mkshort_from_wpt(mkshort_handle, wpt) : wpt->shortname,
 		wpt->latitude < 0 ? 'S' : 'N',  abs(latint), 60.0 * (fabs(wpt->latitude) - latint), 
 		wpt->longitude < 0 ? 'W' : 'E', abs(lonint), 60.0 * (fabs(wpt->longitude) - lonint),
 		utmz, utmzc, utme, utmn);
@@ -117,8 +118,13 @@ text_disp(const waypoint *wpt)
 		fprintf (file_out, "%s\n", wpt->notes);
 	}
 
-	if ( includelogs && wpt->gpx_extras ) {
-		xml_tag *root = wpt->gpx_extras;
+	fs_gpx = NULL;
+	if ( includelogs ) {
+		fs_gpx = (fs_xml *)fs_chain_find( wpt->fs, FS_GPX);
+	}
+	
+	if ( fs_gpx && fs_gpx->tag ) {
+		xml_tag *root = fs_gpx->tag;
 		xml_tag *curlog = NULL;
 		xml_tag *logpart = NULL;
 		curlog = xml_findfirst( root, "groundspeak:log" );
@@ -220,6 +226,7 @@ data_write(void)
 
 ff_vecs_t text_vecs = {
 	ff_type_file,
+	{ ff_cap_write, ff_cap_none, ff_cap_none},
 	NULL,
 	wr_init,
 	NULL,
