@@ -121,7 +121,7 @@ XSTRDUP(const char *s, DEBUG_PARAMS )
 xstrdup(const char *s)
 #endif
 {
-	char *o = strdup(s);
+	char *o = s ? strdup(s) : strdup("");
 #ifdef DEBUG_MEM
 	debug_mem_output( "strdup, %x, %x, %s, %d\n", 
 			o, s, file, line );
@@ -326,6 +326,20 @@ case_ignore_strcmp(const char *s1, const char *s2)
 
 }
 
+int 
+case_ignore_strncmp(const char *s1, const char *s2, int n)
+{
+	int rv = 0;
+
+	while (n && ((rv = toupper(*s1) - toupper(*s2)) == 0)
+		&& *s1) {
+		s1++;
+		s2++;
+		n--;
+	}
+	return rv;
+}
+
 void
 printposn(const double c, int is_lat)
 {
@@ -463,6 +477,7 @@ si_round( double d )
 /*
  *  Return a time_t suitable for adding to a time_t that is in GMT to
  *  make it a local time.
+ *  Obsolete: to use mkgmtime instead.
  */
 signed int 
 get_tz_offset(void)
@@ -477,14 +492,60 @@ get_tz_offset(void)
 	}
 }
 
+/*
+	mkgmtime -- convert tm struct in UTC to time_t
+
+	works just like mktime but without all the mucking
+	around with timezones and daylight savings
+
+	obsoletes get_tz_offset()
+
+	Borrowed from lynx GPL source code
+	http://lynx.isc.org/release/lynx2-8-5/src/mktime.c
+
+	Written by Philippe De Muyter <phdm@macqel.be>.
+*/
+
+time_t
+mkgmtime(struct tm *t)
+{
+	short  month, year;
+	time_t result;
+	static int      m_to_d[12] =
+		{0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
+
+	month = t->tm_mon;
+	year = t->tm_year + month / 12 + 1900;
+	month %= 12;
+	if (month < 0)
+	{
+		year -= 1;
+		month += 12;
+	}
+	result = (year - 1970) * 365 + m_to_d[month];
+	if (month <= 1)
+		year -= 1;
+	result += (year - 1968) / 4;
+	result -= (year - 1900) / 100;
+	result += (year - 1600) / 400;
+	result += t->tm_mday;
+	result -= 1;
+	result *= 24;
+	result += t->tm_hour;
+	result *= 60;
+	result += t->tm_min;
+	result *= 60;
+	result += t->tm_sec;
+	return(result);
+}
+
+
 /*
  * A wrapper for time(2) that allows us to "freeze" time for testing.
  */
 time_t
 current_time(void)
 {
-	static char *frozen;
-	
 	if (getenv("GPSBABEL_FREEZE_TIME")) {
 		return 0;
 	}
@@ -621,7 +682,7 @@ double degrees2ddmm(double deg_val) {
  * Doesn't try to make an optimally sized dest buffer.
  */
 char *
-strsub(char *s, char *search, char *replace)
+strsub(const char *s, const char *search, const char *replace)
 {
        char *p;
        int len = strlen(s);
@@ -649,6 +710,23 @@ strsub(char *s, char *search, char *replace)
        return d;
 }
 
+/*
+ *  As strsub, but do it globally.
+ */
+char *
+gstrsub(const char *s, const char *search, const char *replace)
+{
+	char *o = xstrdup(s);
+
+	while (strstr(o, search)) {
+		char *oo = o;
+		o = strsub(o, search, replace);
+		xfree(oo);
+	}
+
+	return o;
+}
+
 char *
 rot13( const char *s )
 {
@@ -776,13 +854,39 @@ char * str_utf8_to_cp1252( const char * str )
 				}
 			}
 			*cur = (char)value;
-			strcpy( cur+1, cur+bytes );
+			memmove(cur+1, cur+bytes, strlen(cur+bytes) + 1);
 		}
 		cur++;
 	}
 	return result;
 }
 
+#if 0
+/*
+ * Convert to ISO 8859-1 (LATIN-1). The result is never longer than 
+ * the source.  
+ */
+char * 
+str_utf8_to_8859_1( const char * str )
+{
+	char *result = xstrdup( str );
+	char *cur = result;
+	unsigned char c;
+
+	while (c = *str++) {
+		if (c < 0x80) {
+			*cur++ = c;
+			continue;
+		} 
+		if ((c & 0xFE) == 0xC2) {
+			*cur++ = ((c & 0x03) << 6) | (*str++ & 0x3f);
+		}
+	}
+
+	return result;
+}
+#endif
+
 char * str_utf8_to_ascii( const char * str )
 {
 	char *result;
@@ -840,7 +944,7 @@ char * str_utf8_to_ascii( const char * str )
 				cur += bytes - 1;
 			} else {
 				*cur = (char)value;
-				strcpy( cur+1, cur+bytes );
+				memmove(cur+1, cur+bytes, strlen(cur+bytes));
 			}
 		}
 		cur++;
@@ -848,6 +952,48 @@ char * str_utf8_to_ascii( const char * str )
 	return result;
 }
 
+/*
+ * str_iso8859_1_to_utf8
+ *
+ * converts the single byte charset ISO8859-1 (latin1) to UTF-8
+ */
+
+char *
+str_iso8859_1_to_utf8(const char *s)
+{
+	int len;
+	char *res;
+	unsigned char c;
+	char *src, *dst;
+
+	if (s == NULL) return NULL;
+
+	len = 0;
+	src = (char *)s;
+	while ('\0' != (c = *src++))
+	{
+	    len++;
+	    if (c & 0x80) len++;
+	}
+
+	src = (char *)s;
+	dst = res = (void *) xmalloc(len + 1);
+	while ('\0' != (c = *src++))
+	{
+	    if (c & 0x80)
+	    {
+		*dst++ = (0xc0 | (c >> 6));
+		*dst++ = (c & 0xbf);
+	    }
+	    else
+	    {
+		*dst++ = c;
+	    }
+	}
+	*dst = '\0';
+	return res;
+}
+
 /* 
  * Get rid of potentially nasty HTML that would influence another record
  * that includes;
@@ -860,7 +1006,6 @@ strip_nastyhtml(const char * in)
 {
 	char *returnstr, *sp;
 	char *lcstr, *lcp;
-	int i;
 	
 	sp = returnstr = xstrdup(in);
 	lcp = lcstr = xstrdup(in);
@@ -918,7 +1063,7 @@ strip_html(const utf_string *in)
 	short int taglen;
 	
 	if (!in->is_html)
-		return in->utfstring;
+		return xstrdup(in->utfstring);
 	/*
 	 * We only shorten, so just dupe the input buf for space.
 	 */
